import numpy

from PyQt5.QtWidgets import QMessageBox, QLabel, QSizePolicy

from orangewidget.settings import Setting
from orangewidget import gui

from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence
from oasys.util.oasys_util import TriggerIn, TriggerOut, EmittingStream
from oasys.util.oasys_util import write_surface_file, read_surface_file
from oasys.util.oasys_objects import OasysSurfaceData

from syned.widget.widget_decorator import WidgetDecorator

from orangecontrib.wofry.util.wofry_objects import WofryData

from orangecontrib.esrf.wofry.widgets.gui.ow_optical_element_1d import OWWOOpticalElement1D
from orangecontrib.esrf.wofry.util.thin_object import WOThinObject1D #TODO from wofryimpl....



class OWWOThinObject1D(OWWOOpticalElement1D):

    name = "ThinObject1D"
    description = "Wofry: Thin Object 1D"
    icon = "icons/thinb1d.png"
    priority = 6

    inputs = [("WofryData", WofryData, "set_input"),
              ("DABAM 1D Profile", numpy.ndarray, "receive_dabam_profile"),
              ("Trigger", TriggerOut, "receive_trigger_signal"),
              WidgetDecorator.syned_input_data()[0]]

    material = Setting(1)
    refraction_index_delta = Setting(5.3e-7)
    att_coefficient = Setting(0.00357382)

    aperture_shape = Setting(0)
    aperture_dimension_v = Setting(100e-6)
    aperture_dimension_h = Setting(200e-6)


    write_profile_flag = Setting(0)
    write_profile = Setting("thin_object_profile_2D.h5")

    file_with_thickness_mesh = Setting("<none>")

    def __init__(self):

        super().__init__(is_automatic=True, show_view_options=True, show_script_tab=True)

    def draw_specific_box(self):

        self.thinobject_box = oasysgui.widgetBox(self.tab_bas, "Thin Object 1D Setting", addSpace=False, orientation="vertical",
                                           height=350)

        gui.comboBox(self.thinobject_box, self, "material", label="Lens material",
                     items=self.get_material_name(),callback=self.set_visible,
                     sendSelectedValue=False, orientation="horizontal")

        self.box_refraction_index_id = oasysgui.widgetBox(self.thinobject_box, "", addSpace=False, orientation="vertical")
        tmp = oasysgui.lineEdit(self.box_refraction_index_id, self, "refraction_index_delta", "Refraction index delta",
                          labelWidth=250, valueType=float, orientation="horizontal")
        tmp.setToolTip("refraction_index_delta")

        self.box_att_coefficient_id = oasysgui.widgetBox(self.thinobject_box, "", addSpace=False, orientation="horizontal")
        tmp = oasysgui.lineEdit(self.box_att_coefficient_id, self, "att_coefficient", "Attenuation coefficient [m-1]",
                          labelWidth=250, valueType=float, orientation="horizontal")
        tmp.setToolTip("att_coefficient")



        filein_box = oasysgui.widgetBox(self.thinobject_box, "", addSpace=True,
                                        orientation="horizontal")  # width=550, height=50)
        self.le_beam_file_name = oasysgui.lineEdit(filein_box, self, "file_with_thickness_mesh",
                                                   "File with thickness mesh",
                                                   labelWidth=90, valueType=str, orientation="horizontal")
        gui.button(filein_box, self, "...", callback=self.selectFile)


        self.set_visible()

    def set_visible(self):
        self.box_refraction_index_id.setVisible(self.material in [0])
        self.box_att_coefficient_id.setVisible(self.material in [0])

    def selectFile(self):
        filename = oasysgui.selectFileFromDialog(self,
                previous_file_path=self.file_with_thickness_mesh, message="HDF5 Files (*.hdf5 *.h5 *.hdf)",
                start_directory=".", file_extension_filter="*.*")

        self.le_beam_file_name.setText(filename)

    def get_material_name(self, index=None):
        materials_list = ["External", "Be", "Al", "Diamond"]
        if index is None:
            return materials_list
        else:
            return materials_list[index]

    def get_optical_element(self):

        return WOThinObject1D(name=self.oe_name,
                    file_with_thickness_mesh=self.file_with_thickness_mesh,
                    material=self.get_material_name(self.material),
                    refraction_index_delta=self.refraction_index_delta,
                    att_coefficient=self.att_coefficient)

    def check_data(self):
        super().check_data()
        congruence.checkFileName(self.file_with_thickness_mesh)

    def receive_specific_syned_data(self, optical_element):
        pass

    def receive_dabam_profile(self, dabam_profile):
        if not dabam_profile is None:
            try:
                file_name = "dabam_profile_" + str(id(self)) + ".dat"

                file = open(file_name, "w")

                for element in dabam_profile:
                    file.write(str(element[0]) + " " + str(element[1]) + "\n")

                file.flush()
                file.close()

                self.file_with_thickness_mesh = file_name

            except Exception as exception:
                QMessageBox.critical(self, "Error", exception.args[0], QMessageBox.Ok)

                if self.IS_DEVELOP: raise exception

    def receive_trigger_signal(self, trigger):

        if trigger and trigger.new_object == True:
            if trigger.has_additional_parameter("variable_name"):
                variable_name = trigger.get_additional_parameter("variable_name").strip()
                variable_display_name = trigger.get_additional_parameter("variable_display_name").strip()
                variable_value = trigger.get_additional_parameter("variable_value")
                variable_um = trigger.get_additional_parameter("variable_um")

                if "," in variable_name:
                    variable_names = variable_name.split(",")

                    for variable_name in variable_names:
                        setattr(self, variable_name.strip(), variable_value)
                else:
                    setattr(self, variable_name, variable_value)

                self.propagate_wavefront()



    #
    # overwritten methods to append profile plot
    #

    def get_titles(self):
        titles = super().get_titles()
        titles.append("O.E. Profile")
        return titles

    # def propagate_wavefront(self):
    #     super().propagate_wavefront()
    #
    #     if self.write_profile_flag == 1:
    #         xx, yy, s = self.get_optical_element().get_surface_thickness_mesh(self.input_data.get_wavefront())
    #         write_surface_file(s.T, xx, yy, self.write_profile, overwrite=True)
    #         print("\nFile for OASYS " + self.write_profile + " written to disk.")

    def do_plot_results(self, progressBarValue=80): # OVERWRITTEN

        super().do_plot_results(progressBarValue)
        if not self.view_type == 0:
            if not self.wavefront_to_plot is None:

                self.progressBarSet(progressBarValue)

                wo_lens = self.get_optical_element()
                abscissas_on_lens, lens_thickness = wo_lens.get_surface_thickness_mesh(self.input_data.get_wavefront())

                self.plot_data1D(x=abscissas_on_lens*1e6, #TODO check how is possible to plot both refractive surfaces
                                 y=lens_thickness*1e6, # in microns
                                 progressBarValue=progressBarValue + 10,
                                 tabs_canvas_index=4,
                                 plot_canvas_index=4,
                                 calculate_fwhm=False,
                                 title=self.get_titles()[4],
                                 xtitle="Spatial Coordinate along o.e. [$\mu$m]",
                                 ytitle="Total lens thickness [$\mu$m]")

                self.progressBarFinished()

if __name__ == "__main__":
    import sys
    from PyQt5.QtWidgets import QApplication

    def get_example_wofry_data():
        from wofryimpl.propagator.light_source import WOLightSource
        from wofryimpl.beamline.beamline import WOBeamline
        from orangecontrib.wofry.util.wofry_objects import WofryData

        light_source = WOLightSource(dimension=1,
                                     initialize_from=0,
                                     range_from_h=-0.0003,
                                     range_to_h=0.0003,
                                     # range_from_v=-0.0003,
                                     # range_to_v=0.0003,
                                     number_of_points_h=400,
                                     # number_of_points_v=200,
                                     energy=10000.0,
                                     )

        return WofryData(wavefront=light_source.get_wavefront(),
                           beamline=WOBeamline(light_source=light_source))



    a = QApplication(sys.argv)
    ow = OWWOThinObject1D()
    ow.file_with_thickness_mesh = "/users/srio/Oasys/dabam_profile_139821049876720.dat"
    ow.set_input(get_example_wofry_data())


    ow.show()
    a.exec_()
    ow.saveSettings()



import os, sys

from PyQt5 import QtGui, QtWidgets
from PyQt5.QtGui import QPalette, QColor, QFont
from PyQt5.QtWidgets import QApplication, QFileDialog

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence

from orangecontrib.esrf.shadow.util.python_script import PythonConsole

from orangecontrib.shadow.util.shadow_objects import ShadowBeam
from orangecontrib.shadow.util.shadow_util import ShadowCongruence, ShadowPlot
from oasys.widgets import widget
from PyQt5.QtCore import QRect

import inspect
import numpy
import Shadow




class ShadowPythonScript(widget.OWWidget):

    name = "Shadow Python Script"
    description = "Shadow Python Script"
    icon = "icons/python_script.png"
    maintainer = "Manuel Sanchez del Rio"
    maintainer_email = "srio(@at@)esrf.eu"
    priority = 5
    category = "Tools"
    keywords = ["script"]

    inputs = [("Input Beam", ShadowBeam, "setBeam")]

    input_shadow_data=None

    # sampFactNxNyForProp = Setting(1.0) #0.6 #sampling factor for adjusting nx, ny (effective if > 0)
    # nMacroElec = Setting(500000) #T otal number of Macro-Electrons (Wavefronts)
    # nMacroElecAvgOneProc = Setting(5) # Number of Macro-Electrons (Wavefronts) to average on each node (for MPI calculations)
    # nMacroElecSavePer = Setting(20) # Saving periodicity (in terms of Macro-Electrons) for the Resulting Intensity
    # srCalcMeth = Setting(1) # SR calculation method (1- undulator)
    # srCalcPrec = Setting(0.01) # SR calculation rel. accuracy
    # strIntPropME_OutFileName = Setting("output_srw_script_me.dat")
    # _char = Setting(0)


    script_file_flag = Setting(0)
    script_file_name = Setting("tmp.py")
    source_flag = Setting(0)
    source_file_name = Setting("begin.dat")
    do_run = Setting(1)
    iwrite = Setting(1)

    #
    #
    #
    IMAGE_WIDTH = 890
    IMAGE_HEIGHT = 680

    # want_main_area=1

    is_automatic_run = Setting(True)

    error_id = 0
    warning_id = 0
    info_id = 0

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT = 560

    # srw_live_propagation_mode = "Unknown"




    def __init__(self, show_automatic_box=True, show_general_option_box=True):
        super().__init__() # show_automatic_box=show_automatic_box)


        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width()*0.05),
                               round(geom.height()*0.05),
                               round(min(geom.width()*0.98, self.MAX_WIDTH)),
                               round(min(geom.height()*0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        self.general_options_box = gui.widgetBox(self.controlArea, "General Options", addSpace=True, orientation="horizontal")
        self.general_options_box.setVisible(show_general_option_box)

        if show_automatic_box :
            gui.checkBox(self.general_options_box, self, 'is_automatic_run', 'Automatic Execution')


        #
        #
        #
        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Refresh Script", callback=self.refresh_script)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        # button = gui.button(button_box, self, "Reset Fields", callback=self.callResetSettings)
        # font = QFont(button.font())
        # font.setItalic(True)
        # button.setFont(font)
        # palette = QPalette(button.palette()) # make a copy of the palette
        # palette.setColor(QPalette.ButtonText, QColor('Dark Red'))
        # button.setPalette(palette) # assign new palette
        # button.setFixedHeight(45)
        # button.setFixedWidth(150)

        gui.separator(self.controlArea)

        gen_box = oasysgui.widgetBox(self.controlArea, "Script Generation", addSpace=False, orientation="vertical", height=530, width=self.CONTROL_AREA_WIDTH-5)

        gui.comboBox(gen_box, self, "script_file_flag", label="write file with script",
                     items=["No", "Yes"], labelWidth=300,
                     sendSelectedValue=False, orientation="horizontal")

        box1 = gui.widgetBox(gen_box, orientation="horizontal")
        oasysgui.lineEdit(box1, self, "script_file_name", "Script File Name", labelWidth=150, valueType=str,
                          orientation="horizontal")
        self.show_at("self.script_file_flag == 1", box1)


        gui.comboBox(gen_box, self, "source_flag", label="source from",
                     items=["Oasys wire", "shadow3 file"], labelWidth=300,
                     sendSelectedValue=False, orientation="horizontal")

        box1 = gui.widgetBox(gen_box, orientation="horizontal")
        oasysgui.lineEdit(box1, self, "source_file_name", "Source File Name", labelWidth=150, valueType=str,
                          orientation="horizontal")
        self.show_at("self.source_flag == 1", box1)

        gui.comboBox(gen_box, self, "do_run", label="run shadow3 (in script)",
                     items=["No (only definitions)", "Yes (define and run)"], labelWidth=300,
                     sendSelectedValue=False, orientation="horizontal")

        gui.comboBox(gen_box, self, "iwrite", label="when running, write shadow files",
                     items=["No", "Yes"], labelWidth=300,
                     sendSelectedValue=False, orientation="horizontal")


        tabs_setting = oasysgui.tabWidget(self.mainArea)
        tabs_setting.setFixedHeight(self.IMAGE_HEIGHT)
        tabs_setting.setFixedWidth(self.IMAGE_WIDTH)

        tab_scr = oasysgui.createTabPage(tabs_setting, "Python Script")
        tab_out = oasysgui.createTabPage(tabs_setting, "System Output")

        self.pythonScript = oasysgui.textArea(readOnly=False)
        self.pythonScript.setStyleSheet("background-color: white; font-family: Courier, monospace;")
        self.pythonScript.setMaximumHeight(self.IMAGE_HEIGHT - 250)

        script_box = oasysgui.widgetBox(tab_scr, "", addSpace=False, orientation="vertical", height=self.IMAGE_HEIGHT - 10, width=self.IMAGE_WIDTH - 10)
        script_box.layout().addWidget(self.pythonScript)

        console_box = oasysgui.widgetBox(script_box, "", addSpace=True, orientation="vertical",
                                          height=150, width=self.IMAGE_WIDTH - 10)

        self.console = PythonConsole(self.__dict__, self)
        console_box.layout().addWidget(self.console)

        self.shadow_output = oasysgui.textArea()

        out_box = oasysgui.widgetBox(tab_out, "System Output", addSpace=True, orientation="horizontal", height=self.IMAGE_WIDTH - 45)
        out_box.layout().addWidget(self.shadow_output)

        #############################

        button_box = oasysgui.widgetBox(tab_scr, "", addSpace=True, orientation="horizontal")

        gui.button(button_box, self, "Run Script", callback=self.execute_script, height=40)
        # gui.button(button_box, self, "Save Script to File", callback=self.save_script, height=40)

        gui.rubber(self.controlArea)

        self.process_showers()

    def callResetSettings(self):
        pass

    def execute_script(self):

        self._script = str(self.pythonScript.toPlainText())
        self.console.write("\nRunning script:\n")
        self.console.push("exec(_script)")
        self.console.new_prompt(sys.ps1)


    def save_script(self):
        # file_name = QFileDialog.getSaveFileName(self, "Save File to Disk", os.getcwd(), filter='*.py')[0]
        file_name = self.script_file_name
        if not file_name is None:
            if not file_name.strip() == "":
                file = open(file_name, "w")
                file.write(str(self.pythonScript.toPlainText()))
                file.close()


    def setBeam(self, beam):
        if ShadowCongruence.checkEmptyBeam(beam):
            if ShadowCongruence.checkGoodBeam(beam):
                # sys.stdout = EmittingStream(textWritten=self.writeStdOut)

                self.input_beam = beam

                if self.is_automatic_run:
                    self.refresh_script()

            else:
                QtWidgets.QMessageBox.critical(self, "Error",
                                           "Data not displayable: No good rays or bad content",
                                           QtWidgets.QMessageBox.Ok)

    def refresh_script(self):
        optical_element_list_start = []
        optical_element_list_end = []

        self.pythonScript.setText("")

        for history_element in self.input_beam.getOEHistory():
            if not history_element._shadow_source_start is None:
                optical_element_list_start.append(history_element._shadow_source_start.src)
            elif not history_element._shadow_oe_start is None:
                optical_element_list_start.append(history_element._shadow_oe_start._oe)

            if not history_element._shadow_source_end is None:
                optical_element_list_end.append(history_element._shadow_source_end.src)
            elif not history_element._shadow_oe_end is None:
                optical_element_list_end.append(history_element._shadow_oe_end._oe)


        try:
            if self.script_file_flag == 0:
                script_file = ""
            else:
                script_file = self.script_file_name
            self.pythonScript.setText(make_python_script_from_list(optical_element_list_start,
                                                                   script_file=script_file,
                                                                   source_flag=self.source_flag,
                                                                   source_file_name=self.source_file_name,
                                                                   do_run=self.do_run,
                                                                   iwrite=self.iwrite) )
        except:
            self.pythonScript.setText(
                "Problem in writing python script:\n" + str(sys.exc_info()[0]) + ": " + str(sys.exc_info()[1]))

        if self.script_file_flag:
            self.save_script()


        # if not self.input_shadow_data is None:
        #     self.pythonScript.setText("")
        #
        #     try:
        #         received_light_source = self.input_shadow_data.get_srw_beamline().get_light_source()
        #
        #         if not (isinstance(received_light_source, SRWBendingMagnetLightSource) or isinstance(received_light_source, SRWUndulatorLightSource)):
        #             raise ValueError("ME Script is not available with this source")
        #
        #         _char = 0 if self._char == 0 else 4
        #
        #         parameters = [self.sampFactNxNyForProp,
        #                       self.nMacroElec,
        #                       self.nMacroElecAvgOneProc,
        #                       self.nMacroElecSavePer,
        #                       self.srCalcMeth,
        #                       self.srCalcPrec,
        #                       self.strIntPropME_OutFileName,
        #                       _char]
        #
        #         self.pythonScript.setText(self.input_shadow_data.get_srw_beamline().to_python_code([self.input_shadow_data.get_srw_wavefront(), True, parameters]))
        #     except Exception as e:
        #         self.pythonScript.setText("Problem in writing python script:\n" + str(sys.exc_info()[0]) + ": " + str(sys.exc_info()[1]))
        #
        #         if self.IS_DEVELOP: raise e


#
# automatic creation of python scripts
#

def make_python_script_from_list(list_optical_elements1, script_file="", source_flag=0, source_file_name="",
                                 iwrite=0, do_run=1):
    """
    program to build automatically a python script to run shadow3

    the system is read from a list of instances of Shadow.Source and Shadow.OE

    :argument list of optical_elements A python list with intances of Shadow.Source and Shadow.OE objects
    :param script_file: a string with the name of the output file (default="", no output file)
    :return: template with the script
    """

    # make sure that the list does not contain lists

    haslist = sum([isinstance(i, list) for i in list_optical_elements1])

    list_optical_elements = list_optical_elements1
    if haslist:
        while (haslist > 0):
            newlist = []
            for i in list_optical_elements:
                if isinstance(i, list):
                    newlist.extend(i)
                else:
                    newlist.append(i)
            list_optical_elements = newlist
            haslist = sum([isinstance(i, list) for i in list_optical_elements])

    # make sure that the list does not contain compoundOE (developed)

    hascomp = sum(
        [isinstance(i, (Shadow.CompoundOE, Shadow.ShadowLibExtensions.CompoundOE)) for i in list_optical_elements])

    if hascomp:
        newlist = []
        for i in list_optical_elements:
            if isinstance(i, (Shadow.CompoundOE, Shadow.ShadowLibExtensions.CompoundOE)):
                newlist.extend(i.list)
            else:
                newlist.append(i)
        list_optical_elements = newlist

    template_import = """#
# Python script to run shadow3. Created automatically with make_python_script_from_list().
#
import Shadow
import numpy

"""

    n_elements = len(list_optical_elements)
    #
    # source
    #


    template_define_source = """
def define_source():
    #
    # initialize shadow3 source (oe0) and beam
    #
    oe0 = Shadow.Source()

    # Define variables. See https://raw.githubusercontent.com/oasys-kit/shadow3/master/docs/source.nml
"""

    isource = -1
    for i, element in enumerate(list_optical_elements):
        if isinstance(element, Shadow.Source):
            isource = i

    if isource == -1:
        raise Exception("Source not found")

    ioe = isource
    oe1B = list_optical_elements[isource]
    template_define_source += "\n"
    if isinstance(oe1B, Shadow.Source):
        oe1 = Shadow.Source()


    memB = inspect.getmembers(oe1B)
    mem = inspect.getmembers(oe1)
    for i, var in enumerate(memB):
        ivar = mem[i]
        ivarB = memB[i]
        if ivar[0].isupper():
            if isinstance(ivar[1], numpy.ndarray):

                if not ((ivar[1] == ivarB[1]).all()):
                    line = "    oe" + str(ioe) + "." + ivar[0] + " = numpy.array(" + str(ivarB[1].tolist()) + ")\n"
                    template_define_source += line

            else:
                if ivar[1] != ivarB[1]:
                    if isinstance(ivar[1], (str, bytes)):
                        line = "    oe" + str(ioe) + "." + ivar[0] + " = " + str(ivarB[1]).strip() + "\n"
                        if "SPECIFIED" in line:
                            pass
                        else:
                            template_define_source += line
                    else:
                        line = "    oe" + str(ioe) + "." + ivar[0] + " = " + str(ivarB[1]) + "\n"
                        template_define_source += line

    template_define_source += """
    return oe0
    """

    template_run_source = """
def run_source(oe0, iwrite=False):
    # iwrite (1) or not (0) SHADOW files start.xx end.xx star.xx

    #Run SHADOW to create the source

    if iwrite:
        oe0.write("start.00")

    beam = Shadow.Beam()
    beam.genSource(oe0)

    if iwrite:
        oe0.write("end.00")
        beam.write("begin.dat")

    return beam
"""

    #
    # trace
    #

    template_define_beamline = """

def define_beamline():
    # initialize elements
    oe_list = []

    """

    for i, element in enumerate(list_optical_elements):
        if isinstance(element, Shadow.OE):
            template_i = """
    oe{} = Shadow.OE()
    oe_list.append(oe{})"""
            template_define_beamline += template_i.format("%d" % (i), "%d" % (i))
        elif isinstance(element, Shadow.IdealLensOE):
            template_i = """
    oe{} = Shadow.IdealLensOE()
    oe_list.append(oe{})"""
            template_define_beamline += template_i.format("%d" % (i), "%d" % (i))

    template_define_beamline += """

    # Define variables. See https://raw.githubusercontent.com/oasys-kit/shadow3/master/docs/oe.nml
"""

    for ioe, oe1B in enumerate(list_optical_elements):
        template_define_beamline += "\n"
        if isinstance(oe1B, Shadow.Source):
            oe1 = Shadow.Source()
        elif isinstance(element, Shadow.OE):
            oe1 = Shadow.OE()
        elif isinstance(element, Shadow.IdealLensOE):
            oe1 = Shadow.IdealLensOE()

        if ioe != 0:

            if isinstance(oe1B, Shadow.IdealLensOE):
                template_define_beamline += "   oe" + str(ioe) + ".T_SOURCE = " + str(oe1B.T_SOURCE).strip() + "\n"
                template_define_beamline += "   oe" + str(ioe) + ".T_IMAGE = " + str(oe1B.T_IMAGE).strip() + "\n"
                template_define_beamline += "   oe" + str(ioe) + ".focal_x = " + str(oe1B.focal_x).strip() + "\n"
                template_define_beamline += "   oe" + str(ioe) + ".focal_z = " + str(oe1B.focal_z).strip() + "\n"
            else:
                memB = inspect.getmembers(oe1B)
                mem = inspect.getmembers(oe1)
                for i, var in enumerate(memB):
                    ivar = mem[i]
                    ivarB = memB[i]
                    if ivar[0].isupper():
                        if isinstance(ivar[1], numpy.ndarray):
                            # print("                  are ALL different ? ", (ivar[1] != ivarB[1]).all())
                            # print("                  are the same ? ", (ivar[1] == ivarB[1]).all())
                            # print("                  there is at least ONE diff ? ", not((ivar[1] == ivarB[1]).all()))

                            if not ((ivar[1] == ivarB[1]).all()):
                                line = "    oe" + str(ioe) + "." + ivar[0] + " = numpy.array(" + str(
                                    ivarB[1].tolist()) + ")\n"
                                template_define_beamline += line

                            # if (ivar[1] != ivarB[1]).all():
                            #     line = "oe"+str(ioe)+"."+ivar[0]+" = "+str(ivarB[1])+"\n"
                            #     if ("SPECIFIED" in line):
                            #         pass
                            #     else:
                            #         template += line
                        else:
                            if ivar[1] != ivarB[1]:
                                if isinstance(ivar[1], (str, bytes)):
                                    line = "    oe" + str(ioe) + "." + ivar[0] + " = " + str(ivarB[1]).strip() + "\n"
                                    # line = re.sub('\s{2,}', ' ',line)
                                    if "SPECIFIED" in line:
                                        pass
                                    else:
                                        template_define_beamline += line
                                else:
                                    line = "    oe" + str(ioe) + "." + ivar[0] + " = " + str(ivarB[1]) + "\n"
                                    template_define_beamline += line



    template_define_beamline += """\n

    return oe_list

    """

    ###########################

    template_run_beamline = """
def run_beamline(beam_in, oe_list, iwrite=0):
    beam = beam_in.duplicate()
        """

    template_oeA = """
    #
    #run optical element {0}
    #
    print("    Running optical element: %d"%({0}))
    oe{0} = oe_list[{0}-1]
    if iwrite:
        oe{0}.write("start.{1}")
    """

    template_oeB = """
    oe{0} = oe_list[{0}-1]
    if iwrite:
        oe{0}.write("end.{1}")
        beam.write("star.{1}")
"""

    for i in range(1, n_elements):
        template_run_beamline += template_oeA.format(i, "%02d" % (i))
        if isinstance(list_optical_elements[i], Shadow.OE):
            template_run_beamline += "\n" + "    beam.traceOE(oe%d,%d)" % (i, i)
        elif isinstance(list_optical_elements[i], Shadow.IdealLensOE):
            template_run_beamline += "\n" + "    beam.traceIdealLensOE(oe%d,%d)" % (i, i)
        template_run_beamline += template_oeB.format(i, "%02d" % (i))

    template_run_beamline += """
    return beam
"""
    ###########################
    #
    # put together all pieces
    #
    template = ""
    template += template_import

    if source_flag == 0:
        template += template_define_source
        if do_run: template += template_run_source

#         template += """
# oe0 = define_source()
# """

    if n_elements > 1:
        template += template_define_beamline
        if do_run:
            template += template_run_beamline

    #
    # run
    #

    template += """
#
# main
#
"""

    if source_flag == 0:
        template += """
oe0 = define_source()
"""
        if do_run:
            template_i = """
beam = run_source(oe0, iwrite={})
    """
            template += template_i.format("%d" % iwrite)
    elif source_flag == 1:

        template_i = """
beam = Shadow.Beam()
beam.load("{}")
        """
        template += template_i.format("%s" % source_file_name)

    if n_elements > 1:
        template += """
oe_list = define_beamline()
"""
        if do_run:
            template_i = """
beam = run_beamline(beam, oe_list, iwrite={})
"""
            template += template_i.format("%s" % iwrite)

    template += """
# Shadow.ShadowTools.plotxy(beam,1,3,nbins=101,nolost=1,title="Real space")
# Shadow.ShadowTools.plotxy(beam,1,4,nbins=101,nolost=1,title="Phase space X")
# Shadow.ShadowTools.plotxy(beam,3,6,nbins=101,nolost=1,title="Phase space Z")
"""


    if script_file != "":
        open(script_file, "wt").write(template)
        print("File written to disk: %s" % (script_file))

    return template



if __name__ == "__main__":
    import sys
    import Shadow

    class MyBeam():
        pass
    beam_to_analize = Shadow.Beam()
    beam_to_analize.load("/Users/srio/Oasys/screen.0101")
    my_beam = MyBeam()
    my_beam._beam = beam_to_analize

    a = QApplication(sys.argv)
    ow = ShadowPythonScript()
    # ow.pythonScript.setText("#\n#\n#\nprint('Hello world')\n")
    ow.show()
    ow.input_beam = my_beam
    a.exec_()
    ow.saveSettings()


import numpy
import sys

from PyQt5.QtGui import QPalette, QColor, QFont
from PyQt5.QtWidgets import QMessageBox

from orangewidget import gui
from orangewidget import widget
from orangewidget.settings import Setting

from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence
from oasys.util.oasys_util import TriggerIn, TriggerOut, EmittingStream

from syned.storage_ring.magnetic_structures.undulator import Undulator
from syned.beamline.beamline import Beamline
from syned.storage_ring.electron_beam import ElectronBeam
from syned.storage_ring.magnetic_structures.undulator import Undulator


from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D

from wofryimpl.beamline.beamline import WOBeamline
from orangecontrib.esrf.wofry.util.light_source import WOPySRULightSource # TODO: from wofryimpl...

from orangecontrib.wofry.util.wofry_objects import WofryData
from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget

from scipy.ndimage.filters import gaussian_filter1d as gaussian_filter1d


class OWUndulatorPySRU(WofryWidget):

    name = "Undulator pySRU"
    id = "UndulatorPySRU"
    description = "Undulator pySRU"
    icon = "icons/undulator.png"
    priority = 201

    category = "Wofry Wavefront Propagation"
    keywords = ["data", "file", "load", "read"]

    inputs = [
                ("SynedData", Beamline, "receive_syned_data"),
                ("Trigger", TriggerOut, "receive_trigger_signal"),
                ]
    outputs = [
               {"name":"WofryData",
                "type":WofryData,
                "doc":"WofryData",
                "id":"WofryData"}
                ]

    gapH          = Setting(0.001)
    gapV          = Setting(0.001)
    h_slit_points = Setting(51)
    v_slit_points = Setting(51)
    distance      = Setting(30.0)

    sigma_h = Setting(3.01836e-05)
    sigma_v = Setting(3.63641e-06)
    sigma_divergence_h = Setting(4.36821e-06)
    sigma_divergence_v = Setting(1.37498e-06)

    photon_energy = Setting(10000.0)

    period_length = Setting(0.020)
    number_of_periods = Setting(100)
    K_vertical = Setting(1.19)
    electron_energy_in_GeV = Setting(6.0)
    ring_current = Setting(0.2)

    scan_direction = Setting(0)

    flag_send_wavefront_dimension = Setting(0)

    calculated_wavefront = None

    def __init__(self):

        super().__init__(is_automatic=False, show_view_options=True, show_script_tab=True)

        self.runaction = widget.OWAction("Generate Wavefront", self)
        self.runaction.triggered.connect(self.calculate)
        self.addAction(self.runaction)


        gui.separator(self.controlArea)
        gui.separator(self.controlArea)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Calculate", callback=self.calculate)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        gui.separator(self.controlArea)

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        tabs_setting = oasysgui.tabWidget(self.controlArea)
        tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT + 50)
        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)

        self.tab_settings = oasysgui.createTabPage(tabs_setting, "Settings")
        self.tab_lightsource = oasysgui.createTabPage(tabs_setting, "Convolution parameters")

        #
        # Settings
        #
        box_wavefront = oasysgui.widgetBox(self.tab_settings, "Sampling wavefront", addSpace=False, orientation="vertical")



        oasysgui.lineEdit(box_wavefront, self, "distance", "distance from source origin [m]",
                          labelWidth=300, tooltip="distance",
                          valueType=float, orientation="horizontal")

        oasysgui.lineEdit(box_wavefront, self, "gapH", "H aperture [m]",
                          labelWidth=300, tooltip="gapH",
                          valueType=float, orientation="horizontal")

        oasysgui.lineEdit(box_wavefront, self, "gapV", "V aperture [m]",
                          labelWidth=300, tooltip="gapV",
                          valueType=float, orientation="horizontal")

        oasysgui.lineEdit(box_wavefront, self, "h_slit_points", "Number of Points in H",
                          labelWidth=300, tooltip="h_slit_points",
                          valueType=int, orientation="horizontal")

        oasysgui.lineEdit(box_wavefront, self, "v_slit_points", "Number of Points in V",
                          labelWidth=300, tooltip="v_slit_points",
                          valueType=int, orientation="horizontal")

        left_box_33 = oasysgui.widgetBox(box_wavefront, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(left_box_33, self, "photon_energy", "Photon Energy [eV]",
                          labelWidth=200, tooltip="photon_energy",
                          valueType=float, orientation="horizontal")
        gui.button(left_box_33, self, "set from K", callback=self.set_photon_energy, width=80)

        #
        # Light Source
        #
        box_ring = oasysgui.widgetBox(self.tab_settings, "Storage ring", addSpace=False, orientation="vertical")

        oasysgui.lineEdit(box_ring, self, "electron_energy_in_GeV", "Energy [GeV]",  labelWidth=260, valueType=float, orientation="horizontal", callback=self.update)
        oasysgui.lineEdit(box_ring, self, "ring_current", "Ring Current [A]",        labelWidth=260, valueType=float, orientation="horizontal", callback=self.update)

        #
        # Undulator
        #
        box_und = oasysgui.widgetBox(self.tab_settings, "Undulator", addSpace=False, orientation="vertical")

        oasysgui.lineEdit(box_und, self, "period_length", "Period Length [m]", labelWidth=260,
                          valueType=float, orientation="horizontal", callback=self.update)
        oasysgui.lineEdit(box_und, self, "number_of_periods", "Number of Periods", labelWidth=260,
                          valueType=float, orientation="horizontal", callback=self.update)

        oasysgui.lineEdit(box_und, self, "K_vertical", "Vertical K", labelWidth=260,
                          valueType=float, orientation="horizontal")


        box_send = oasysgui.widgetBox(self.tab_settings, "Send wavefront", addSpace=True, orientation="vertical")
        gui.comboBox(box_send, self, "flag_send_wavefront_dimension", label="Send Wavefront", labelWidth=350,
                     items=["2D",
                            "1D Horizontal",
                            "1D Vertical",
                            ],
                     # callback=self.set_visible,
                     sendSelectedValue=False, orientation="horizontal")

        #
        # second tab
        #

        self.emittances_box_h = oasysgui.widgetBox(self.tab_lightsource, "Electron Horizontal beam sizes",
                                            addSpace=True, orientation="vertical")
        self.emittances_box_v = oasysgui.widgetBox(self.tab_lightsource, "Electron Vertical beam sizes",
                                            addSpace=True, orientation="vertical")


        self.le_sigma_h = oasysgui.lineEdit(self.emittances_box_h, self, "sigma_h", "Size RMS H",
                            labelWidth=250, tooltip="sigma_h",
                            valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.emittances_box_h, self, "sigma_divergence_h", "Divergence RMS H [rad]",
                            labelWidth=250, tooltip="sigma_divergence_h",
                            valueType=float, orientation="horizontal")


        self.le_sigma_v = oasysgui.lineEdit(self.emittances_box_v, self, "sigma_v", "Size RMS V",
                            labelWidth=250, tooltip="sigma_v",
                            valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.emittances_box_v, self, "sigma_divergence_v", "Divergence RMS V [rad]",
                            labelWidth=250, tooltip="sigma_divergence_v",
                            valueType=float, orientation="horizontal")





    def initializeTabs(self):
        size = len(self.tab)
        indexes = range(0, size)

        for index in indexes:
            self.tabs.removeTab(size-1-index)

        self.titles = ["Intensity",
                       "Phase",
                       "Intensity with convolution"]
        self.tab = []
        self.plot_canvas = []

        for index in range(0, len(self.titles)):
            self.tab.append(gui.createTabPage(self.tabs, self.titles[index]))
            self.plot_canvas.append(None)

        for tab in self.tab:
            tab.setFixedHeight(self.IMAGE_HEIGHT)
            tab.setFixedWidth(self.IMAGE_WIDTH)

    def set_photon_energy(self):
        light_source = self.get_light_source()
        su = light_source.get_magnetic_structure()
        ebeam = light_source.get_electron_beam()
        self.photon_energy = numpy.round(su.resonance_energy(ebeam.gamma(), harmonic=1.0), 3)

    def check_fields(self):
        congruence.checkStrictlyPositiveNumber(self.photon_energy, "Photon Energy")

        congruence.checkGreaterOrEqualThan(self.distance, self.number_of_periods * self.period_length,
                                           "distance", "undulator length")

        congruence.checkStrictlyPositiveNumber(self.v_slit_points, "Number of Points V")
        congruence.checkStrictlyPositiveNumber(self.h_slit_points, "Number of Points H")

        congruence.checkNumber(self.gapH, "gapH")
        congruence.checkNumber(self.gapV, "gapV")

        congruence.checkStrictlyPositiveNumber(self.electron_energy_in_GeV, "electron_energy_in_GeV")
        congruence.checkNumber(self.ring_current, "ring_current")
        congruence.checkStrictlyPositiveNumber(self.period_length, "period_length")
        congruence.checkStrictlyPositiveNumber(self.number_of_periods, "number_of_periods")
        congruence.checkStrictlyPositiveNumber(self.K_vertical, "K_vertical")


    def receive_syned_data(self, data):
        if not data is None:
            if isinstance(data, Beamline):
                if not data._light_source is None:
                    if isinstance(data._light_source._magnetic_structure, Undulator):
                        light_source = data._light_source
                        ebeam = light_source.get_electron_beam()
                        und = light_source.get_magnetic_structure()

                        x, xp, y, yp = ebeam.get_sigmas_all()
                        self.sigma_h = x
                        self.sigma_v = y
                        self.sigma_divergence_h = xp
                        self.sigma_divergence_v = yp
                        self.electron_energy_in_GeV = ebeam.energy()
                        self.ring_current = ebeam.current()

                        self.number_of_periods = und.number_of_periods()
                        self.period_length = und.period_length()
                        self.photon_energy =  round(und.resonance_energy(ebeam.gamma()), 3)
                        self.K_vertical = und.K_vertical()


                    else:
                        raise ValueError("Syned light source not congruent")
                else:
                    raise ValueError("Syned data not correct: light source not present")
            else:
                raise ValueError("Syned data not correct")

    def receive_trigger_signal(self, trigger):

        if trigger and trigger.new_object == True:
            if trigger.has_additional_parameter("variable_name"):
                variable_name = trigger.get_additional_parameter("variable_name").strip()
                variable_display_name = trigger.get_additional_parameter("variable_display_name").strip()
                variable_value = trigger.get_additional_parameter("variable_value")
                variable_um = trigger.get_additional_parameter("variable_um")

                if "," in variable_name:
                    variable_names = variable_name.split(",")

                    for variable_name in variable_names:
                        setattr(self, variable_name.strip(), variable_value)
                else:
                    setattr(self, variable_name, variable_value)

                self.calculate()

    def get_light_source(self):

        return WOPySRULightSource.initialize_from_keywords(
            name="Undefined",
            energy_in_GeV=self.electron_energy_in_GeV,
            current=self.ring_current,
            K_vertical=self.K_vertical,
            period_length=self.period_length,
            number_of_periods=self.number_of_periods,
            distance=self.distance,
            gapH=self.gapH,
            gapV=self.gapV,
            photon_energy=self.photon_energy,
            h_slit_points=self.h_slit_points,
            v_slit_points=self.v_slit_points,
            flag_send_wavefront_dimension=self.flag_send_wavefront_dimension,
        )

    def do_plot_results(self, progressBarValue):


        #
        # plot intensity
        #
        wf = self.calculated_wavefront
        if isinstance(wf, GenericWavefront2D):
            self.progressBarSet(progressBarValue)
            self.plot_data2D(wf.get_intensity(),
                             1e6 * wf.get_coordinate_x(),
                             1e6 * wf.get_coordinate_y(),
                             20, 0, 0,
                             title=self.titles[0],
                             xtitle="Spatial Coordinate H [$\mu$m]",
                             ytitle="Spatial Coordinate V [$\mu$m]")

            #
            # plot phase
            #
            self.plot_data2D(wf.get_phase(),
                             1e6 * wf.get_coordinate_x(),
                             1e6 * wf.get_coordinate_y(),
                             20, 1, 1,
                             title=self.titles[1],
                             xtitle="Spatial Coordinate H [$\mu$m]",
                             ytitle="Spatial Coordinate V [$\mu$m]")

            #
            # plot convolution
            #

            #
            # convolution for non zero emittance
            #

            intensArray = self.calculated_wavefront.get_intensity()
            wf = self.calculated_wavefront

            hArray = wf.get_coordinate_x()
            vArray = wf.get_coordinate_y()

            SigmaH = numpy.sqrt(self.sigma_h ** 2 + (self.distance * self.sigma_divergence_h) ** 2)
            SigmaV = numpy.sqrt(self.sigma_v ** 2 + (self.distance * self.sigma_divergence_v) ** 2)

            print("\nFor the Convolution Tab, convolving with SigmaH: %g, SigmaV: %g" % ( SigmaH, SigmaV))
            print("\n**NOTE that the Convolution Tab is for info only. This is not considered in the wavefront")
            intensArray = gaussian_filter1d(intensArray, SigmaH / (hArray[1] - hArray[0]), axis=0)
            intensArray = gaussian_filter1d(intensArray, SigmaV / (vArray[1] - vArray[0]), axis=1)


            self.plot_data2D(intensArray,
                             1e6 * hArray,
                             1e6 * vArray,
                             20, 2, 2,
                             title=self.titles[2],
                             xtitle="Spatial Coordinate H [$\mu$m]",
                             ytitle="Spatial Coordinate V [$\mu$m]")

    def calculate(self):

        self.wofry_output.setText("calculating...")

        sys.stdout = EmittingStream(textWritten=self.writeStdOut)


        self.progressBarInit()

        self.check_fields()

        self.progressBarSet(10.0)

        # main calculation
        ls = self.get_light_source()
        self.calculated_wavefront = ls.get_wavefront()

        # ii = self.calculated_wavefront.get_integrated_intensity()
        deltaX = 1e3 * (self.calculated_wavefront.get_coordinate_x()[1] - self.calculated_wavefront.get_coordinate_x()[0])
        deltaY = 1e3 * (self.calculated_wavefront.get_coordinate_y()[1] - self.calculated_wavefront.get_coordinate_y()[0])

        pixel_angle = deltaX * deltaY / self.distance**2
        print("\n\ndeltaX: %g mm, deltaY: %g mm, distance: %g m" % (deltaX, deltaY, self.distance))
        print("Pixel angle: %g mrad2" % pixel_angle)
        ii = self.calculated_wavefront.get_intensity().sum()
        print("Integrated intensity (raw): %g " % (ii))
        print("Integrated intensity (normalized): %g photons/s/0.1bw" % (ii * pixel_angle))


        # plots
        if self.view_type != 0:
            self.initializeTabs()
            self.do_plot_results(90.0)

        self.progressBarFinished()

        try:
            beamline = WOBeamline(light_source=ls)
            self.wofry_python_script.set_code(beamline.to_python_code())
        except:
            pass


        # beamline = WOBeamline(light_source=ls)
        print(">>> sending wavefront ")

        if self.flag_send_wavefront_dimension == 0:
            self.send("WofryData", WofryData(
                wavefront=self.calculated_wavefront,
                beamline=WOBeamline(light_source=ls)))
        elif self.flag_send_wavefront_dimension == 1: # H
            self.send("WofryData", WofryData(
                wavefront=self.calculated_wavefront.get_Wavefront1D_from_profile(0, 0.0),
                beamline=WOBeamline(light_source=ls)))
        elif self.flag_send_wavefront_dimension == 2: # V
            self.send("WofryData", WofryData(
                wavefront=self.calculated_wavefront.get_Wavefront1D_from_profile(1, 0.0),
                beamline=WOBeamline(light_source=ls)))


if __name__ == "__main__":
    import sys
    from PyQt5.QtWidgets import QApplication

    a = QApplication(sys.argv)
    ow = OWUndulatorPySRU()

    ow.show()
    a.exec_()
    ow.saveSettings()




from syned.beamline.optical_element import OpticalElement
from syned.beamline.shape import SurfaceShape



class OpticalElementsWithTwoSurfaceShapes(OpticalElement):

    def __init__(self, name="",
                 surface_shape1=None,
                 surface_shape2=None,
                 boundary_shape=None):
        super().__init__(name, boundary_shape)
        if surface_shape1 is None: surface_shape1 = SurfaceShape()
        if surface_shape2 is None: surface_shape1 = SurfaceShape()
        self._surface_shape1 = surface_shape1
        self._surface_shape2 = surface_shape2

    def get_surface_shape(self, index):
        if index == 0:
            return self._surface_shape1
        elif index == 1:
            return self._surface_shape2
        else:
            raise Exception("only two shapes in OpticalElementsWithTwoSurfaceShapes")

    def set_surface_shape(self, index, surface_shape=None):
        if surface_shape is None: surface_shape = SurfaceShape()
        if index == 0:
            self._surface_shape2 = surface_shape
        elif index == 1:
            self._surface_shape2 = surface_shape
        else:
            raise Exception("only two shapes in OpticalElementsWithTwoSurfaceShapes")



import sys
import numpy
from PyQt5.QtWidgets import QApplication, QMessageBox, QSizePolicy

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui, congruence
from oasys.widgets.exchange import DataExchangeObject
from orangecontrib.xoppy.util.xoppy_xraylib_util import xpower_calc

from oasys.widgets.exchange import DataExchangeObject
from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget


import scipy.constants as codata

class OWID19AttenuatorsBox(XoppyWidget):
    name = "ID19AttenuatorsBox"
    id = "orange.widgets.dataxpower"
    description = "Power Absorbed and Transmitted by Optical Elements"
    icon = "icons/id19_attenuator.png"
    priority = 2
    category = ""
    keywords = ["xoppy", "power"]

    inputs = [("ExchangeData", DataExchangeObject, "acceptExchangeData")]

    FLAG = Setting(0)
    SOURCE = Setting(2)
    ENER_MIN = Setting(1000.0)
    ENER_MAX = Setting(50000.0)
    ENER_N = Setting(100)
    SOURCE_FILE = Setting("?")
    ATT11 = Setting(0)
    ATT12 = Setting(3)
    ATT13 = Setting(3)
    ATT14 = Setting(3)
    ATT15 = Setting(3)
    ATT21 = Setting(3)
    ATT22 = Setting(3)
    ATT23 = Setting(3)
    ATT24 = Setting(3)
    ATT25 = Setting(3)
    C = Setting(1.4)
    Al = Setting(1.4)
    Cu = Setting(0.0)
    Mo = Setting(0.0)
    W = Setting(0.0)
    Au = Setting(0.14)
    PLOT_SETS = Setting(2)
    FILE_DUMP = 0


    def build_gui(self):

        self.leftWidgetPart.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))
        self.leftWidgetPart.setMaximumWidth(self.CONTROL_AREA_WIDTH + 20)
        self.leftWidgetPart.updateGeometry()

        box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)

        idx = -1


        # widget index 1
        idx += 1
        box1 = gui.widgetBox(box)
        self.box_source = gui.comboBox(box1, self, "FLAG",
                                       label=self.unitLabels()[idx], addSpace=False,
                                       items=['Attenuators Box', 'Attenuators list'],
                                       valueType=int, orientation="horizontal", labelWidth=150, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)


        #widget index 2 
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.box_source = gui.comboBox(box1, self, "SOURCE",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['From Oasys wire', 'Normalized to 1', 'From external file.                '],
                    valueType=int, orientation="horizontal", labelWidth=150)
        self.show_at(self.unitFlags()[idx], box1)
        
        #widget index 6 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_MIN",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 7 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_MAX",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 8 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_N",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 


        # widget index 9 ***********   File Browser ******************
        idx += 1
        box1 = gui.widgetBox(box)
        file_box_id = oasysgui.widgetBox(box1, "", addSpace=False, orientation="horizontal")
        self.file_id = oasysgui.lineEdit(file_box_id, self, "SOURCE_FILE", self.unitLabels()[idx],
                                         labelWidth=100, valueType=str, orientation="horizontal")
        gui.button(file_box_id, self, "...", callback=self.select_input_file, width=25)
        self.show_at(self.unitFlags()[idx], box1)


        #widget index 10
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT11",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['101 : None', '102 : Diam, 1.4mm','103 : Diam, 2.8mm', '104 : Diam, 1mm'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)



        #widget index 11
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT12",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['201 : Al, 0.7mm', '202 : Al, 1.4mm','203 : Al, 2.8mm', '204 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 12
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT13",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['301 : Cu, 4mm', '302 : Cu, 6mm','303 : Cu, 8mm', '304 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 13
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT14",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['401 : Al, 0.7mm', '402 : Al, 1.4mm','403 : Al, 2.8mm', '404 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 14
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT15",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['501 : Cu, 0.14mm', '502 : Cu, 0.35mm','503 : Cu, 1.4mm', '504 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 15
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT21",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['601 : Cu, 0.7mm', '602 : Cu, 1.4mm','603 : Cu, 2.8mm', '604 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 16
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT22",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['701 : Mo, 0.14mm', '702 : Mo, 0.35mm','703 : Mo, 0.7mm', '704 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 17
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT23",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['801 : W, 0.3mm', '802 : W, 0.5mm','803 : W, 1.0mm', '804 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 18
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT24",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['901 : W, 0.07mm', '902 : W, 0.14mm','903 : W, 0.28mm', '904 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 19
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "ATT25",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['1001 : Au, 0.07mm', '1002 : Au, 0.14mm','1003 : Au, 0.28mm', '1004 : None'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 20
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "C",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 21
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "Al",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 22
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "Cu",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 23
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "Mo",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 24
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "W",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 25
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "Au",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 41
        idx += 1
        box1 = gui.widgetBox(box)
        gui.separator(box1, height=7)

        gui.comboBox(box1, self, "PLOT_SETS",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['Local properties', 'Cumulated intensity', 'All'],
                    valueType=int, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 42
        idx += 1
        box1 = gui.widgetBox(box)
        gui.separator(box1, height=7)

        gui.comboBox(box1, self, "FILE_DUMP",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['No', 'Yes (power.spec)'],
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        self.input_spectrum = None





    def select_input_file(self):
        self.file_id.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE,
                                    "Open 2-columns file with spectral power",
                                    file_extension_filter="ascii dat (*.dat *.txt *spec)"))



    def set_EL_FLAG(self):
        self.initializeTabs()

    def unitLabels(self):
         return ['Box or List','Input beam:',
                 'From energy [eV]:      ',
                 'To energy [eV]:',
                 'Energy points:  ',
                 'File with input beam spectral power:',
                 'Att11','Att12','Att13','Att14','Att15','Att21','Att22','Att23','Att24','Att25',
                 'thickness of the C attenuator (mm) [diamond]', 'thickness of the Al attenuator (mm)',
                 'thickness of the Cu attenuator (mm)', 'thickness of the Mo attenuator (mm)',
                 'thickness of the W attenuator (mm)', 'thickness of the Au attenuator (mm)',
                 'Plot','Dump file']


    def unitFlags(self):
         return ['True','True',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  2',
                 'self.FLAG  ==  0','self.FLAG  ==  0','self.FLAG  ==  0','self.FLAG  ==  0','self.FLAG  ==  0','self.FLAG  ==  0','self.FLAG  ==  0','self.FLAG  ==  0','self.FLAG  ==  0','self.FLAG  ==  0',
                 'self.FLAG  ==  1','self.FLAG  ==  1','self.FLAG  ==  1','self.FLAG  ==  1','self.FLAG  ==  1','self.FLAG  ==  1',
                 'True','True']

    def get_help_name(self):
        return 'ID19AttenuatorsBox'

    def selectFile(self):
        self.le_source_file.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE, "Open Source File", file_extension_filter="*.*"))

    def acceptExchangeData(self, exchangeData):

        self.input_spectrum = None
        self.SOURCE = 0
        # self.box_source.setCurrentIndex(self.SOURCE)

        try:
            if not exchangeData is None:
                if exchangeData.get_program_name() == "XOPPY":
                    no_bandwidth = False
                    if exchangeData.get_widget_name() =="UNDULATOR_FLUX" :
                        # self.SOURCE_FILE = "xoppy_undulator_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() == "BM" :
                        if exchangeData.get_content("is_log_plot") == 1:
                            raise Exception("Logaritmic X scale of Xoppy Energy distribution not supported")
                        if exchangeData.get_content("calculation_type") == 0 and exchangeData.get_content("psi") == 0:
                            # self.SOURCE_FILE = "xoppy_bm_flux"
                            no_bandwidth = True
                            index_flux = 6
                        else:
                            raise Exception("Xoppy result is not an Flux vs Energy distribution integrated in Psi")
                    elif exchangeData.get_widget_name() =="XWIGGLER" :
                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="WS" :
                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="XTUBES" :
                        # self.SOURCE_FILE = "xoppy_xtubes_flux"
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="XTUBE_W" :
                        # self.SOURCE_FILE = "xoppy_xtube_w_flux"
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="BLACK_BODY" :
                        # self.SOURCE_FILE = "xoppy_black_body_flux"
                        no_bandwidth = True
                        index_flux = 2

                    elif exchangeData.get_widget_name() =="UNDULATOR_RADIATION" :
                        # self.SOURCE_FILE = "xoppy_undulator_radiation"
                        no_bandwidth = True
                        index_flux = 1
                    elif exchangeData.get_widget_name() =="POWER" :
                        # self.SOURCE_FILE = "xoppy_undulator_power"
                        no_bandwidth = True
                        index_flux = -1
                    elif exchangeData.get_widget_name() =="POWER3D" :
                        # self.SOURCE_FILE = "xoppy_power3d"
                        no_bandwidth = True
                        index_flux = 1

                    else:
                        raise Exception("Xoppy Source not recognized")

                    # self.SOURCE_FILE += "_" + str(id(self)) + ".dat"


                    spectrum = exchangeData.get_content("xoppy_data")

                    if exchangeData.get_widget_name() =="UNDULATOR_RADIATION" or \
                        exchangeData.get_widget_name() =="POWER3D":
                        [p, e, h, v ] = spectrum
                        tmp = p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*codata.e*1e3
                        spectrum = numpy.vstack((e,p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*
                                                 codata.e*1e3))
                        self.input_spectrum = spectrum
                    else:

                        if not no_bandwidth:
                            spectrum[:,index_flux] /= 0.001*spectrum[:,0]

                        self.input_spectrum = numpy.vstack((spectrum[:,0],spectrum[:,index_flux]))

                    self.process_showers()
                    self.compute()

        except Exception as exception:
            QMessageBox.critical(self, "Error",
                                       str(exception),
                QMessageBox.Ok)

            #raise exception




    def check_fields(self):

        if self.SOURCE == 1:
            self.ENER_MIN = congruence.checkPositiveNumber(self.ENER_MIN, "Energy from")
            self.ENER_MAX = congruence.checkStrictlyPositiveNumber(self.ENER_MAX, "Energy to")
            congruence.checkLessThan(self.ENER_MIN, self.ENER_MAX, "Energy from", "Energy to")
            self.NPOINTS = congruence.checkStrictlyPositiveNumber(self.ENER_N, "Energy Points")
        elif self.SOURCE == 2:
            congruence.checkFile(self.SOURCE_FILE)
        if self.FLAG == 1:
            self.C=congruence.checkPositiveNumber(self.C,'thickness of the C attenuator (mm)')
            self.Al = congruence.checkPositiveNumber(self.Al, 'thickness of the Al attenuator (mm)')
            self.Cu = congruence.checkPositiveNumber(self.Cu, 'thickness of the Cu attenuator (mm)')
            self.Mo = congruence.checkPositiveNumber(self.Mo, 'thickness of the Mo attenuator (mm)')
            self.W = congruence.checkPositiveNumber(self.W, 'thickness of the W attenuator (mm)')
            self.Au = congruence.checkPositiveNumber(self.Au, 'thickness of the Au attenuator (mm)')


    def do_xoppy_calculation(self):
        return self.xoppy_calc_xpower()

    def extract_data_from_xoppy_output(self, calculation_output):
        return calculation_output


    def get_data_exchange_widget_name(self):
        return "POWER"

    def do_plot_local(self):
        out = False
        if self.PLOT_SETS == 0: out = True
        if self.PLOT_SETS == 2: out = True
        return out

    def do_plot_intensity(self):
        out = False
        if self.PLOT_SETS == 1: out = True
        if self.PLOT_SETS == 2: out = True
        return out


    def getTitles(self):
        titles = []

        if self.do_plot_intensity(): titles.append("Input beam")
        #if self.do_plot_local(): titles.append("Total CS")
        #if self.do_plot_local(): titles.append("Mu")
        if self.do_plot_local(): titles.append("Transmitivity")
        if self.do_plot_local(): titles.append("Absorption")
        if self.do_plot_intensity(): titles.append("Intensity")
        return titles

    def getXTitles(self):

        xtitles = []

        if self.do_plot_intensity(): xtitles.append("Photon Energy [eV]")
        #if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
        #if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
        if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
        if self.do_plot_local(): xtitles.append("Photon Energy [eV]")
        if self.do_plot_intensity(): xtitles.append("Photon Energy [eV]")
        return xtitles

    def getYTitles(self):
        ytitles = []
        if self.do_plot_intensity(): ytitles.append("Source")
        #if self.do_plot_local(): ytitles.append(" Total CS cm2/g")
        #if self.do_plot_local(): ytitles.append(" Mu cm^-1")
        if self.do_plot_local(): ytitles.append(" Transmitivity")
        if self.do_plot_local(): ytitles.append(" Absorption")
        if self.do_plot_intensity(): ytitles.append("Intensity")

        return ytitles

    def getVariablesToPlot(self):
        variables = []

        if self.do_plot_intensity(): variables.append((0, 1))  # start plotting the source
        #if self.do_plot_local(): variables.append((0, 2))
        #if self.do_plot_local(): variables.append((0, 3))
        if self.do_plot_local(): variables.append((0, 2))
        if self.do_plot_local(): variables.append((0, 3))
        if self.do_plot_intensity(): variables.append((0, 4))

        return variables

    def getLogPlot(self):
        logplot = []

        if self.do_plot_intensity(): logplot.append((False,False))
        #if self.do_plot_local(): logplot.append((False, True))
        #if self.do_plot_local(): logplot.append((False, True))
        if self.do_plot_local(): logplot.append((False, False))
        if self.do_plot_local(): logplot.append((False, False))
        if self.do_plot_intensity(): logplot.append((False, False))

        return logplot




    def Attenuators_Thickness(self):
        thick=[]
        if self.FLAG == 0 :
            if self.ATT11== 0: thick.append(0.0)
            if self.ATT11== 1: thick.append(1.4)
            if self.ATT11== 2: thick.append(2.8)
            if self.ATT11== 3: thick.append(1.0)
            if self.ATT12== 0: thick.append(0.7)
            if self.ATT12== 1: thick.append(1.4)
            if self.ATT12== 2: thick.append(2.8)
            if self.ATT12== 3: thick.append(0.0)
            if self.ATT13== 0: thick.append(4.0)
            if self.ATT13== 1: thick.append(6.0)
            if self.ATT13== 2: thick.append(8.0)
            if self.ATT13== 3: thick.append(0.0)
            if self.ATT14== 0: thick.append(0.7)
            if self.ATT14== 1: thick.append(1.4)
            if self.ATT14== 2: thick.append(2.8)
            if self.ATT14== 3: thick.append(0.0)
            if self.ATT15== 0: thick.append(0.14)
            if self.ATT15== 1: thick.append(0.35)
            if self.ATT15== 2: thick.append(1.4)
            if self.ATT15== 3: thick.append(0.0)
            if self.ATT21== 0: thick.append(0.7)
            if self.ATT21== 1: thick.append(1.4)
            if self.ATT21== 2: thick.append(2.8)
            if self.ATT21== 3: thick.append(0.0)
            if self.ATT22== 0: thick.append(0.14)
            if self.ATT22== 1: thick.append(0.35)
            if self.ATT22== 2: thick.append(0.7)
            if self.ATT22== 3: thick.append(0.0)
            if self.ATT23== 0: thick.append(0.3)
            if self.ATT23== 1: thick.append(0.5)
            if self.ATT23== 2: thick.append(1.0)
            if self.ATT23== 3: thick.append(0.0)
            if self.ATT24== 0: thick.append(0.07)
            if self.ATT24== 1: thick.append(0.14)
            if self.ATT24== 2: thick.append(0.28)
            if self.ATT24== 3: thick.append(0.0)
            if self.ATT25== 0: thick.append(0.07)
            if self.ATT25== 1: thick.append(0.14)
            if self.ATT25== 2: thick.append(0.28)
            if self.ATT25== 3: thick.append(0.0)
        if self.FLAG == 1:
            thick=[self.C,self.Al,self.Cu,self.Mo,self.W,self.Au]

        return(thick)



    def xoppy_calc_xpower(self):

        #
        # prepare input for xpower_calc
        # Note that the input for xpower_calc accepts any number of elements.
        #
        if self.FLAG == 0:
            substance = ['C','Al','Cu','Al','Cu','Cu','Mo','W','W','Au']
            thick     = self.Attenuators_Thickness()
            dens      = [3.508,2.7,8.96,2.7,8.96,8.96,10.20,19.3,19.3,19.3]
            flags     =  [0,0,0,0,0,0,0,0,0,0]
        if self.FLAG == 1:
            substance = ['C', 'Al', 'Cu','Mo', 'W','Au']
            thick = self.Attenuators_Thickness()
            dens = [3.508, 2.7, 8.96, 10.20, 19.3, 19.3]
            flags = [0, 0, 0, 0, 0, 0]

        cumulated_data = {}
        Result=[]
        Result_Absorption=[]

        if self.SOURCE == 0:
            if self.input_spectrum is None:
                raise Exception("No input beam")
            else:
                energies = self.input_spectrum[0,:].copy()
                source = self.input_spectrum[1,:].copy()
        elif self.SOURCE == 1:
            energies = numpy.linspace(self.ENER_MIN,self.ENER_MAX,self.ENER_N)
            source = numpy.ones(energies.size)
            tmp = numpy.vstack( (energies,source))
            self.input_spectrum = source
        elif self.SOURCE == 2:
            if self.SOURCE == 2: source_file = self.SOURCE_FILE
            try:
                tmp = numpy.loadtxt(source_file)
                energies = tmp[:,0]
                source = tmp[:,1]
                self.input_spectrum = source
            except:
                print("Error loading file %s "%(source_file))
                raise

        if self.FILE_DUMP == 0:
            output_file = None
        else:
            output_file = "power.spec"
        out_dictionary = xpower_calc(energies=energies,source=source,substance=substance,
                                     flags=flags,dens=dens,thick=thick,angle=[],roughness=[],output_file=output_file)


        try:
            print(out_dictionary["info"])
        except:
            pass
        #calculate attenuators total
        Result.append((out_dictionary['data'][0]).tolist())
        Result.append((out_dictionary['data'][1]).tolist())
        #list2=[]
        #list3=[]
        list4=[]
        for k in range(len(substance)):
            #list2.append(out_dictionary['data'][2 + 5*k])
            #list3.append(out_dictionary['data'][3 + 5*k])
            list4.append(out_dictionary['data'][4 + 5*k])
        #Result.append(List_Product(list2))
        #Result.append(List_Product(list3))
        Result.append(List_Product(list4))

        for k in range(len(Result[0])):
            Result_Absorption.append(1-Result[-1][k])
        Result.append(Result_Absorption)

        Result.append((out_dictionary['data'][5*len(substance)+1]).tolist())
        cumulated_data['data']=numpy.array(Result)

        #send exchange
        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
        try:
            calculated_data.add_content("xoppy_data", cumulated_data["data"].T)
        except:
            pass
        return calculated_data

def List_Product(list):
    L = []
    l = 1
    for k in range(len(list[0])):
        for i in range(len(list)):
            l = l * list[i][k]
        L.append(l)
        l = 1
    return (L)

if __name__ == "__main__":


    from oasys.widgets.exchange import DataExchangeObject



    input_data_type = "POWER"

    if input_data_type == "POWER":
        # create fake UNDULATOR_FLUX xoppy exchange data
        e = numpy.linspace(1000.0, 10000.0, 100)
        source = e/10
        received_data = DataExchangeObject("XOPPY", "POWER")
        received_data.add_content("xoppy_data", numpy.vstack((e,e,source)).T)
        received_data.add_content("xoppy_code", "US")

    elif input_data_type == "POWER3D":
        # create unulator_radiation xoppy exchange data
        from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_radiation

        e, h, v, p, code = xoppy_calc_undulator_radiation(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
                                           ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
                                           ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
                                           PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,
                                           SETRESONANCE=0,HARMONICNUMBER=1,
                                           GAPH=0.001,GAPV=0.001,\
                                           HSLITPOINTS=41,VSLITPOINTS=41,METHOD=0,
                                           PHOTONENERGYMIN=7000,PHOTONENERGYMAX=8100,PHOTONENERGYPOINTS=20,
                                           USEEMITTANCES=1)
        received_data = DataExchangeObject("XOPPY", "POWER3D")
        received_data = DataExchangeObject("XOPPY", "UNDULATOR_RADIATION")
        received_data.add_content("xoppy_data", [p, e, h, v])
        received_data.add_content("xoppy_code", code)




    app = QApplication(sys.argv)
    w = OWID19AttenuatorsBox()
    w.acceptExchangeData(received_data)
    w.show()
    app.exec()
    w.saveSettings()



import sys
import numpy
from PyQt5.QtWidgets import QApplication, QMessageBox, QSizePolicy

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui, congruence
from oasys.widgets.exchange import DataExchangeObject
from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget


import scipy.constants as codata



from crystalpy.diffraction.GeometryType import BraggDiffraction, LaueDiffraction
from crystalpy.diffraction.DiffractionSetup import DiffractionSetup
from crystalpy.diffraction.Diffraction import Diffraction
import scipy.constants as codata
from crystalpy.util.Vector import Vector
from crystalpy.util.Photon import Photon


class Monochromator(XoppyWidget):
    name = "Monochromator"
    id = "orange.widgets.dataxpower"
    description = "Power Absorbed and Transmitted by Optical Elements"
    icon = "icons/id19_monochromator.png"
    priority = 3
    category = ""
    keywords = ["xoppy", "power", "monochromator"]

    inputs = [("ExchangeData", DataExchangeObject, "acceptExchangeData")]

    SOURCE = Setting(2)
    TYPE = Setting(0)
    ENER_SELECTED = (8000)
    # HARMONIC = Setting (15000)
    H_MILLER = Setting (1)
    K_MILLER = Setting (1)
    L_MILLER = Setting (1)
    THICK = (15)
    ENER_MIN = Setting(10000.0)
    ENER_MAX = Setting(200000.0)
    ENER_N = Setting(2000)
    SOURCE_FILE = Setting("?")
    FILE_DUMP = 0

    def build_gui(self):

        self.leftWidgetPart.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))
        self.leftWidgetPart.setMaximumWidth(self.CONTROL_AREA_WIDTH + 20)
        self.leftWidgetPart.updateGeometry()

        box_main = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)

        idx = -1

        box = oasysgui.widgetBox(box_main, "Input Beam Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)
        # widget index 1
        idx += 1
        box1 = gui.widgetBox(box)
        self.box_source = gui.comboBox(box1, self, "SOURCE",
                                       label=self.unitLabels()[idx], addSpace=False,
                                       items=['From Oasys wire', 'Normalized to 1',
                                              'From external file.                '],
                                       valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)


        # widget index 6
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ENER_MIN",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 7
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ENER_MAX",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 8
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ENER_N",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 9 ***********   File Browser ******************
        idx += 1
        box1 = gui.widgetBox(box)
        file_box_id = oasysgui.widgetBox(box1, "", addSpace=False, orientation="horizontal")
        self.file_id = oasysgui.lineEdit(file_box_id, self, "SOURCE_FILE", self.unitLabels()[idx],
                                         labelWidth=100, valueType=str, orientation="horizontal")
        gui.button(file_box_id, self, "...", callback=self.select_input_file, width=25)
        self.show_at(self.unitFlags()[idx], box1)





        box = oasysgui.widgetBox(box_main, "Monochromator", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)
        # widget index 2
        idx += 1
        box1 = gui.widgetBox(box)
        self.box_source = gui.comboBox(box1, self, "TYPE",
                                       label=self.unitLabels()[idx], addSpace=False,
                                       items=['Empty','Bragg (double reflection)','Laue (single reflection)','Multilayer'],
                                       valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 3
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ENER_SELECTED",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 4.1
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "H_MILLER",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 4.2
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "K_MILLER",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 4.3
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "L_MILLER",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)



        # widget index 5
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "THICK",
                                       label=self.unitLabels()[idx], addSpace=False,
                                       valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)


        #widget index 10
        idx += 1
        box1 = gui.widgetBox(box)
        gui.separator(box1, height=7)

        gui.comboBox(box1, self, "FILE_DUMP",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['No', 'Yes (monochromator.spec)'],
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        self.input_spectrum = None




    def select_input_file(self):
        self.file_id.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE,
                                    "Open 2-columns file with spectral power",
                                    file_extension_filter="ascii dat (*.dat *.txt *spec)"))



    def unitLabels(self):
         return ['Input beam:',
                 'From energy [eV]:      ',
                 'To energy [eV]:',
                 'Energy points:  ',
                 'File with input beam spectral power:',
                 'Type Monochromator',
                 'Energy Selected [eV]',
                 'miller index h','miller index k','miller index l','Crystal thickness [microns]',
                 "Dump file"]


    def unitFlags(self):
         return ['True',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  2',
                 'True',
                 'self.TYPE  ==  1 or self.TYPE  ==  2 or self.TYPE  ==  3',
                 'self.TYPE  ==  1 or self.TYPE  ==  2','self.TYPE  ==  1 or self.TYPE  ==  2','self.TYPE  ==  1 or self.TYPE  ==  2',
                 'self.TYPE  ==  2',
                 'True']

    def get_help_name(self):
        return 'Monochromator'

    def selectFile(self):
        self.le_source_file.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE, "Open Source File", file_extension_filter="*.*"))

    def acceptExchangeData(self, exchangeData):

        self.input_spectrum = None
        self.SOURCE = 0
        # self.box_source.setCurrentIndex(self.SOURCE)

        try:
            if not exchangeData is None:
                if exchangeData.get_program_name() == "XOPPY":
                    no_bandwidth = False
                    if exchangeData.get_widget_name() =="UNDULATOR_FLUX" :
                        # self.SOURCE_FILE = "xoppy_undulator_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() == "BM" :
                        if exchangeData.get_content("is_log_plot") == 1:
                            raise Exception("Logaritmic X scale of Xoppy Energy distribution not supported")
                        if exchangeData.get_content("calculation_type") == 0 and exchangeData.get_content("psi") == 0:
                            # self.SOURCE_FILE = "xoppy_bm_flux"
                            no_bandwidth = True
                            index_flux = 6
                        else:
                            raise Exception("Xoppy result is not an Flux vs Energy distribution integrated in Psi")
                    elif exchangeData.get_widget_name() =="XWIGGLER" :
                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="WS" :
                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="XTUBES" :
                        # self.SOURCE_FILE = "xoppy_xtubes_flux"
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="XTUBE_W" :
                        # self.SOURCE_FILE = "xoppy_xtube_w_flux"
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="BLACK_BODY" :
                        # self.SOURCE_FILE = "xoppy_black_body_flux"
                        no_bandwidth = True
                        index_flux = 2

                    elif exchangeData.get_widget_name() =="UNDULATOR_RADIATION" :
                        # self.SOURCE_FILE = "xoppy_undulator_radiation"
                        no_bandwidth = True
                        index_flux = 1
                    elif exchangeData.get_widget_name() =="POWER" :
                        # self.SOURCE_FILE = "xoppy_undulator_power"
                        no_bandwidth = True
                        index_flux = -1
                    elif exchangeData.get_widget_name() =="POWER3D" :
                        # self.SOURCE_FILE = "xoppy_power3d"
                        no_bandwidth = True
                        index_flux = 1

                    else:
                        raise Exception("Xoppy Source not recognized")

                    # self.SOURCE_FILE += "_" + str(id(self)) + ".dat"

                    spectrum = exchangeData.get_content("xoppy_data")

                    if exchangeData.get_widget_name() =="UNDULATOR_RADIATION" or \
                        exchangeData.get_widget_name() =="POWER3D":
                        [p, e, h, v ] = spectrum
                        tmp = p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*codata.e*1e3
                        spectrum = numpy.vstack((e,p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*
                                                 codata.e*1e3))
                        self.input_spectrum = spectrum
                    else:

                        if not no_bandwidth:
                            spectrum[:,index_flux] /= 0.001*spectrum[:,0]

                        self.input_spectrum = numpy.vstack((spectrum[:,0],spectrum[:,index_flux]))

                    self.process_showers()
                    self.compute()

        except Exception as exception:
            QMessageBox.critical(self, "Error",
                                       str(exception),
                QMessageBox.Ok)

            #raise exception




    def check_fields(self):
        if self.TYPE  ==  1:
            self.ENER_SELECTED = congruence.checkPositiveNumber(self.ENER_SELECTED, "Energy Selected [eV]")
            self.H_MILLER = congruence.checkNumber(self.H_MILLER, "H Miller")
            self.K_MILLER = congruence.checkNumber(self.K_MILLER, "K Miller")
            self.L_MILLER = congruence.checkNumber(self.H_MILLER, "L Miller")
        if self.TYPE == 2:
            self.ENER_SELECTED = congruence.checkPositiveNumber(self.ENER_SELECTED, "Energy Selected [eV]")
            self.H_MILLER = congruence.checkNumber(self.H_MILLER, "H Miller")
            self.K_MILLER = congruence.checkNumber(self.K_MILLER, "K Miller")
            self.L_MILLER = congruence.checkNumber(self.H_MILLER, "L Miller")
            self.THICK = congruence.checkPositiveNumber(self.THICK, "Laue crystal thickness [mm]")
        if self.TYPE == 3:
            self.ENER_SELECTED = congruence.checkPositiveNumber(self.ENER_SELECTED, "Energy Selected [eV]")

        if self.SOURCE == 1:
            self.ENER_MIN = congruence.checkPositiveNumber(self.ENER_MIN, "Energy from")
            self.ENER_MAX = congruence.checkStrictlyPositiveNumber(self.ENER_MAX, "Energy to")
            congruence.checkLessThan(self.ENER_MIN, self.ENER_MAX, "Energy from", "Energy to")
            self.NPOINTS = congruence.checkStrictlyPositiveNumber(self.ENER_N, "Energy Points")
        elif self.SOURCE == 2:
            congruence.checkFile(self.SOURCE_FILE)

    def do_xoppy_calculation(self):
        return self.xoppy_calc_mono()

    def extract_data_from_xoppy_output(self, calculation_output):
        return calculation_output


    def get_data_exchange_widget_name(self):
        return "POWER"




    def getTitles(self):
        return ['Input Beam','Monochromator reflectivity','Transmitted intensity']

    def getXTitles(self):
        return ["Energy [eV]","Energy [eV]","Energy [eV]"]

    def getYTitles(self):
        return ["Source",'Reflectivity','Intensity']


    def getVariablesToPlot(self):
        return [(0, 1),(0, 2),(0, 3)]

    def getLogPlot(self):
        return [(False,False),(False, False),(False, False)]


    def calculate_bragg_dcm(self, h_miller=1, k_miller=1, l_miller=1,
                            energy_setup=8000.0, energies=numpy.linspace(7900, 8100, 200)):

        energy_setup = self.ENER_SELECTED
        r = numpy.zeros_like(energies)
        harmonic = 1

        diffraction_setup_r = DiffractionSetup(geometry_type=BraggDiffraction(),  # GeometryType object
                                               crystal_name="Si",  # string
                                               thickness=1,  # meters
                                               miller_h=harmonic * h_miller,  # int
                                               miller_k=harmonic * k_miller,  # int
                                               miller_l=harmonic * l_miller,  # int
                                               asymmetry_angle=0,  # 10.0*numpy.pi/180.,            # radians
                                               azimuthal_angle=0.0)  # radians                            # int


        bragg_angle = diffraction_setup_r.angleBragg(energy_setup)
        print("Bragg angle for Si%d%d%d at E=%f eV is %f deg" % (
            h_miller, k_miller, l_miller, energy_setup, bragg_angle * 180.0 / numpy.pi))
        nharmonics = int( energies.max() / energy_setup)

        if nharmonics < 1:
            nharmonics = 1
        print("Calculating %d harmonics" % nharmonics)

        for harmonic in range(1,nharmonics+1):
            for i in range(energies.size):
                try:
                    diffraction_setup_r = DiffractionSetup(geometry_type=BraggDiffraction(),  # GeometryType object
                                                           crystal_name="Si",  # string
                                                           thickness=1,  # meters
                                                           miller_h=harmonic * h_miller,  # int
                                                           miller_k=harmonic * k_miller,  # int
                                                           miller_l=harmonic * l_miller,  # int
                                                           asymmetry_angle=0,  # 10.0*numpy.pi/180.,            # radians
                                                           azimuthal_angle=0.0)  # radians                            # int

                    diffraction = Diffraction()

                    energy = energies[i]
                    deviation = 0.0  # angle_deviation_min + ia * angle_step
                    angle = deviation + bragg_angle

                    # calculate the components of the unitary vector of the incident photon scan
                    # Note that diffraction plane is YZ
                    yy = numpy.cos(angle)
                    zz = - numpy.abs(numpy.sin(angle))
                    photon = Photon(energy_in_ev=energy, direction_vector=Vector(0.0, yy, zz))

                    # perform the calculation
                    coeffs_r = diffraction.calculateDiffractedComplexAmplitudes(diffraction_setup_r, photon)
                    # note the power 4 to get intensity (**2) for a double reflection (**2)
                    r[i] += numpy.abs(coeffs_r['S'].complexAmplitude()) ** 4
                except:
                    print("Failed to calculate reflectivity at E=%g eV for %d%d%d reflection" % (energy,
                                            harmonic*h_miller, harmonic*k_miller, harmonic*l_miller))

        return r

    def calculate_laue_monochromator(self, h_miller=1, k_miller=1, l_miller=1,
                            energy_setup=8000.0, energies=numpy.linspace(7900, 8100, 200)):

        energy_setup = self.ENER_SELECTED
        r = numpy.zeros_like(energies)
        harmonic = 1
        diffraction_setup_r = DiffractionSetup(geometry_type=LaueDiffraction(),  # GeometryType object
                                               crystal_name="Si",  # string
                                               thickness=self.THICK*1e-6,  # meters
                                               miller_h=harmonic * h_miller,  # int
                                               miller_k=harmonic * k_miller,  # int
                                               miller_l=harmonic * l_miller,  # int
                                               asymmetry_angle=numpy.pi/2,  # 10.0*numpy.pi/180.,            # radians
                                               azimuthal_angle=0)  # radians                            # int


        bragg_angle = diffraction_setup_r.angleBragg(energy_setup)
        print("Bragg angle for Si%d%d%d at E=%f eV is %f deg" % (
            h_miller, k_miller, l_miller, energy_setup, bragg_angle * 180.0 / numpy.pi))
        nharmonics = int( energies.max() / energy_setup)

        if nharmonics < 1:
            nharmonics = 1
        print("Calculating %d harmonics" % nharmonics)

        for harmonic in range(1,nharmonics+1):
            for i in range(energies.size):
                try:
                    diffraction_setup_r = DiffractionSetup(geometry_type=LaueDiffraction(),  # GeometryType object
                                                           crystal_name="Si",  # string
                                                           thickness=self.THICK*1e-6,  # meters
                                                           miller_h=harmonic * h_miller,  # int
                                                           miller_k=harmonic * k_miller,  # int
                                                           miller_l=harmonic * l_miller,  # int
                                                           asymmetry_angle=numpy.pi/2,  # 10.0*numpy.pi/180.,            # radians
                                                           azimuthal_angle=0)  # radians                            # int

                    diffraction = Diffraction()

                    energy = energies[i]
                    deviation = 0.0  # angle_deviation_min + ia * angle_step
                    angle = deviation + numpy.pi/2 + bragg_angle

                    # calculate the components of the unitary vector of the incident photon scan
                    # Note that diffraction plane is YZ
                    yy = numpy.cos(angle)
                    zz = - numpy.abs(numpy.sin(angle))
                    photon = Photon(energy_in_ev=energy, direction_vector=Vector(0.0, yy, zz))

                    # perform the calculation
                    coeffs_r = diffraction.calculateDiffractedComplexAmplitudes(diffraction_setup_r, photon)
                    # note the power 2 to get intensity
                    r[i] += numpy.abs(coeffs_r['S'].complexAmplitude()) ** 2
                except:
                    print("Failed to calculate reflectivity at E=%g eV for %d%d%d reflection" % (energy,
                                            harmonic*h_miller, harmonic*k_miller, harmonic*l_miller))

        return r
    def xoppy_calc_mono(self):


        if self.SOURCE == 0:
            if self.input_spectrum is None:
                raise Exception("No input beam")
            else:
                energies = self.input_spectrum[0,:].copy()
                source = self.input_spectrum[1,:].copy()
        elif self.SOURCE == 1:
            energies = numpy.linspace(self.ENER_MIN,self.ENER_MAX,self.ENER_N)
            source = numpy.ones(energies.size)
            tmp = numpy.vstack( (energies,source))
            self.input_spectrum = source
        elif self.SOURCE == 2:
            if self.SOURCE == 2: source_file = self.SOURCE_FILE
            try:
                tmp = numpy.loadtxt(source_file)
                energies = tmp[:,0]
                source = tmp[:,1]
                self.input_spectrum = source
            except:
                print("Error loading file %s "%(source_file))
                raise


        # Mono_Effect=[]
        if self.TYPE==0:
            Mono_Effect = [1] * len(energies)
            # for k in range(len(energies)):
            #     Mono_Effect.append(1)
        elif self.TYPE==1:
            Mono_Effect = self.calculate_bragg_dcm(energies=energies)
        elif self.TYPE==2:
            Mono_Effect = self.calculate_laue_monochromator(energies=energies)
        else: #TODO:
            Mono_Effect = [1] * len(energies)




        # from srxraylib.plot.gol import plot
        # plot(energies, Mono_Effect)


        Final_Spectrum=List_Product([source,Mono_Effect])

        Output=[]
        Output.append(energies.tolist())
        Output.append(source.tolist())
        Output.append(Mono_Effect)
        Output.append(Final_Spectrum)
        Output=numpy.array(Output)

        if self.FILE_DUMP == 1:
            output_file = "monochromator.spec"
            f = open(output_file, 'w')
            f.write("#S 1\n#N 4\n#L Photon energy [eV]  source  monochromator reflectivity  transmitted intensity\n")
            for i in range(Output.shape[1]):
                f.write("%g  %g  %g  %g\n" % (Output[0,i], Output[1,i], Output[2,i], Output[3,i]) )
            f.close()
            print("File written to disk: %s" % output_file)

        # print results
        I1 = numpy.trapz( numpy.array(source), x=energies, axis=-1)
        I2 = numpy.trapz( numpy.array(Final_Spectrum), x=energies, axis=-1)
        txt  = "      Incoming power: %f\n"%(I1)
        txt += "      Outcoming power: %f\n" % (I2)
        txt += "      Absorbed power: %f\n"%(I1-I2)
        txt += "      Normalized Absorbed power: %f\n"%((I1-I2)/I1)
        print(txt)


        # send exchange
        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
        try:
            calculated_data.add_content("xoppy_data", Output.T)
        except:
            pass


        return calculated_data

def List_Product(list):
    L = []
    l = 1
    for k in range(len(list[0])):
        for i in range(len(list)):
            l = l * list[i][k]
        L.append(l)
        l = 1
    return (L)

if __name__ == "__main__":


    from oasys.widgets.exchange import DataExchangeObject



    input_data_type = "POWER"

    if input_data_type == "POWER":
        # create fake UNDULATOR_FLUX xoppy exchange data
        e = numpy.linspace(7900.0, 8100.0, 1500)
        source = e/10
        received_data = DataExchangeObject("XOPPY", "POWER")
        received_data.add_content("xoppy_data", numpy.vstack((e,e,source)).T)
        received_data.add_content("xoppy_code", "US")

    elif input_data_type == "POWER3D":
        # create unulator_radiation xoppy exchange data
        from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_radiation

        e, h, v, p, code = xoppy_calc_undulator_radiation(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
                                           ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
                                           ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
                                           PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,
                                           SETRESONANCE=0,HARMONICNUMBER=1,
                                           GAPH=0.001,GAPV=0.001,\
                                           HSLITPOINTS=41,VSLITPOINTS=41,METHOD=0,
                                           PHOTONENERGYMIN=7000,PHOTONENERGYMAX=8100,PHOTONENERGYPOINTS=20,
                                           USEEMITTANCES=1)
        received_data = DataExchangeObject("XOPPY", "POWER3D")
        received_data = DataExchangeObject("XOPPY", "UNDULATOR_RADIATION")
        received_data.add_content("xoppy_data", [p, e, h, v])
        received_data.add_content("xoppy_code", code)




    app = QApplication(sys.argv)
    w = Monochromator()
    w.acceptExchangeData(received_data)
    w.show()
    app.exec()
    w.saveSettings()



import numpy

from oasys.util.oasys_util import write_surface_file
from orangecontrib.esrf.wofry.util.thin_object import WOThinObject, WOThinObject1D #TODO from wofryimpl....
from wofry.beamline.decorators import OpticalElementDecorator

class WOThinObjectCorrector(WOThinObject, OpticalElementDecorator):

    def __init__(self, name="Undefined",
                 file_with_thickness_mesh="",
                 material="",
                 refraction_index_delta=1e-07,
                 att_coefficient=0.0,
                 correction_method=1,
                 focus_at=10.0,
                 wall_thickness=0.0,
                 apply_correction_to_wavefront=0,
                 file_with_thickness_mesh_flag=0,
                 ):

        super().__init__(name=name,
                 file_with_thickness_mesh=file_with_thickness_mesh,
                 material=material,
                         )

        self._correction_method = correction_method
        self._focus_at = focus_at
        self._wall_thickness = wall_thickness
        self._apply_correction_to_wavefront = apply_correction_to_wavefront
        self._file_with_thickness_mesh_flag = file_with_thickness_mesh_flag

        self._refraction_index_delta = refraction_index_delta
        self._att_coefficient = att_coefficient

    def calculate_correction_profile(self, wavefront):

        photon_energy = wavefront.get_photon_energy()

        x = wavefront.get_coordinate_x()
        y = wavefront.get_coordinate_y()

        if self._correction_method == 0: # write file with zero profile
            profile = numpy.zeros((x.size, y.size))
        elif self._correction_method == 1: # focus to waist

            print("\n\n\n ==========  parameters from optical element : ")
            print(self.info())


            refraction_index_delta, att_coefficient = self.get_refraction_index(photon_energy)
            # auxiliar spherical wavefront
            wavefront_model = wavefront.duplicate()
            wavefront_model.set_spherical_wave(radius=-self._focus_at, complex_amplitude=1.0,)


            phase_correction = numpy.angle( wavefront_model.get_complex_amplitude() / wavefront.get_complex_amplitude())
            profile = -phase_correction / wavefront.get_wavenumber() / refraction_index_delta


        profile += self._wall_thickness
        if self._file_with_thickness_mesh_flag:
            write_surface_file(profile.T, x, y, self.get_file_with_thickness_mesh(), overwrite=True)
            print("\nFile for OASYS " + self.get_file_with_thickness_mesh() + " written to disk.")

        # for info


        # H profile
        n = profile.shape[0]
        one_over_fraction_in_length = 10
        w = n // (2 * one_over_fraction_in_length)

        profile_line = profile[:, w]
        xx = x[(n // 2 - w):(n // 2 + w)]
        yy = profile_line[(n // 2 - w):(n // 2 + w)]

        yder = numpy.gradient(yy, xx)
        coeff = numpy.polyfit(xx, yder, 1)
        print("\n\n\n ==========  fitted radius in the H profile center (over 1/%d of length): " % one_over_fraction_in_length)
        print("fitted lens (with two curved sides) of radius = %g m " % (2 / coeff[0]))
        print("which corresponds to a focal length of %g m " % (1 / coeff[0] / refraction_index_delta))

        # V profile
        n = profile.shape[1]
        one_over_fraction_in_length = 10
        w = n // (2 * one_over_fraction_in_length)

        profile_line = profile[w, :]
        xx = y[(n // 2 - w):(n // 2 + w)]
        yy = profile_line[(n // 2 - w):(n // 2 + w)]

        yder = numpy.gradient(yy, xx)
        coeff = numpy.polyfit(xx, yder, 1)
        print("\n\n\n ==========  fitted radius in the V profile center (over 1/%d of length): " % one_over_fraction_in_length)
        print("fitted lens (with two curved sides) of radius = %g m " % (2 / coeff[0]))
        print("which corresponds to a focal length of %g m " % (1 / coeff[0] / refraction_index_delta))

        return profile, x, y


    def applyOpticalElement(self, wavefront, parameters=None, element_index=None):

        profile, x, y = self.calculate_correction_profile(wavefront)

        if self._apply_correction_to_wavefront > 0:
            #TODO change this....
            output_wavefront = super().applyOpticalElement(wavefront, parameters=parameters, element_index=element_index)
        else:
            output_wavefront = wavefront

        return output_wavefront


    def to_python_code(self, data=None):
        txt = ""
        txt += "\nfrom orangecontrib.esrf.wofry.util.thin_object_corrector import WOThinObjectCorrector #TODO update"
        txt += "\n"
        txt += "\noptical_element = WOThinObjectCorrector("
        txt += "\n    name='%s'," % self.get_name()
        txt += "\n    file_with_thickness_mesh_flag=%d," % self._file_with_thickness_mesh_flag
        txt += "\n    file_with_thickness_mesh='%s'," % self.get_file_with_thickness_mesh()
        txt += "\n    material='%s'," % self.get_material()
        if self.get_material() == "External":
            txt += "\n    refraction_index_delta=%g," % self._refraction_index_delta
            txt += "\n    att_coefficient=%g," % self._att_coefficient
        txt += "\n    focus_at=%g," % self._focus_at
        txt += "\n    wall_thickness=%g," % self._wall_thickness
        txt += "\n    apply_correction_to_wavefront=%d)" % self._apply_correction_to_wavefront

        txt += "\n"
        return txt

class WOThinObjectCorrector1D(WOThinObject1D, OpticalElementDecorator):
    def __init__(self, name="Undefined",
                 file_with_thickness_mesh="",
                 material="",
                 refraction_index_delta=1e-07,
                 att_coefficient=0.0,
                 correction_method=1,
                 focus_at=10.0,
                 wall_thickness=0.0,
                 apply_correction_to_wavefront=0,
                 file_with_thickness_mesh_flag=0,
                 fit_fraction_in_length=0.1,
                 fit_filename="",
                 ):

        super().__init__(name=name,
                         file_with_thickness_mesh=file_with_thickness_mesh,
                         material=material,
                         refraction_index_delta=refraction_index_delta,
                         att_coefficient=att_coefficient,
                         )

        self._correction_method = correction_method
        self._focus_at = focus_at
        self._wall_thickness = wall_thickness
        self._apply_correction_to_wavefront = apply_correction_to_wavefront
        self._file_with_thickness_mesh_flag = file_with_thickness_mesh_flag

        self._fit_fraction_in_length = fit_fraction_in_length
        self._fit_filename = fit_filename

    def calculate_correction_profile(self, wavefront):
        photon_energy = wavefront.get_photon_energy()

        x = wavefront.get_abscissas()

        if self._correction_method == 0:  # write file with zero profile
            profile = numpy.zeros_like(x)
            profile += self._wall_thickness
        elif self._correction_method == 1:  # focus to waist

            print("\n\n\n ==========  parameters from optical element : ")
            print(self.info())

            refraction_index_delta, att_coefficient = self.get_refraction_index(photon_energy)
            # auxiliar spherical wavefront
            target_wavefront = wavefront.duplicate()
            target_wavefront.set_spherical_wave(radius=-self._focus_at, complex_amplitude=1.0, )

            phase_input = wavefront.get_phase(unwrap=True)
            phase_target = target_wavefront.get_phase(unwrap=True)
            phase_correction = phase_target - phase_input

            profile = - phase_correction / (wavefront.get_wavenumber() * refraction_index_delta)
            profile -= profile.min()
            profile += self._wall_thickness

        if self._file_with_thickness_mesh_flag:
            f = open(self.get_file_with_thickness_mesh(), 'w')
            for i in range(x.size):
                f.write("%g %g\n" % (x[i], profile[i]))
            f.close()
            print("\nFile 1D for OASYS " + self.get_file_with_thickness_mesh() + " written to disk.")

        # for info
        n = profile.size
        fraction_in_length = self._fit_fraction_in_length
        w = int((n * fraction_in_length) / 2)

        if w <= 1: w = 1


        xx = x[(n // 2 - w):(n // 2 + w)]
        yy = profile[(n // 2 - w):(n // 2 + w)]

        yder = numpy.gradient(yy, xx)
        coeff = numpy.polyfit(xx, yder, 1)
        print("\n\n\n ==========  fitted radius in the profile center (over %g of length): " % fraction_in_length)
        print("fitted lens (with two curved sides) of radius = %g m " % (2 / coeff[0]))
        print("which corresponds to a focal length of %g m " % (1 / coeff[0] / refraction_index_delta))

        if self._fit_filename != "":
            f = open(self._fit_filename, 'w')
            f.write("# ==========  fitted radius in the profile center (over %g of length): \n" % fraction_in_length)
            f.write("# fitted lens (with two curved sides) of radius = %g m \n" % (2 / coeff[0]))
            f.write("# which corresponds to a focal length of %g m \n" % (1 / coeff[0] / refraction_index_delta))

            f.write("%g\n" % (2 / coeff[0]))
            f.write("%g\n" % (1 / coeff[0] / refraction_index_delta))
            f.close()
            print("File %s written to disk." % self._fit_filename)
        return x, profile

    def applyOpticalElement(self, wavefront, parameters=None, element_index=None):

        x, profile = self.calculate_correction_profile(wavefront)

        refraction_index_delta, att_coefficient = self.get_refraction_index(wavefront.get_photon_energy())

        if self._apply_correction_to_wavefront > 0:

            amp_factors = numpy.exp(-1.0 * att_coefficient * profile / 2) # factor of 2 because it is amplitude
            phase_shifts = -1.0 * wavefront.get_wavenumber() * refraction_index_delta * profile

            output_wavefront = wavefront.duplicate()
            output_wavefront.rescale_amplitudes(amp_factors)
            output_wavefront.add_phase_shifts(phase_shifts)
        else:
            output_wavefront = wavefront

        return output_wavefront

    def to_python_code(self, data=None):
        txt = ""
        txt += "\nfrom orangecontrib.esrf.wofry.util.thin_object_corrector import WOThinObjectCorrector1D #TODO update"
        txt += "\n"
        txt += "\noptical_element = WOThinObjectCorrector1D("
        txt += "\n    name='%s'," % self.get_name()
        txt += "\n    file_with_thickness_mesh_flag=%d," % self._file_with_thickness_mesh_flag
        txt += "\n    file_with_thickness_mesh='%s'," % self.get_file_with_thickness_mesh()
        txt += "\n    material='%s'," % self.get_material()
        if self.get_material() == "External":
            txt += "\n    refraction_index_delta=%g," % self._refraction_index_delta
            txt += "\n    att_coefficient=%g," % self._att_coefficient
        txt += "\n    focus_at=%g," % self._focus_at
        txt += "\n    wall_thickness=%g," % self._wall_thickness
        txt += "\n    apply_correction_to_wavefront=%d," % self._apply_correction_to_wavefront
        txt += "\n    fit_fraction_in_length=%g," % self._fit_fraction_in_length
        txt += "\n    fit_filename='%s')" % self._fit_filename

        txt += "\n"
        return txt






# class ALSFiniteElementReader(ALSShadowWidget):

import os, sys
import numpy

from PyQt5 import QtGui, QtWidgets
from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QRect, Qt

from PyQt5.QtWidgets import QLabel, QSizePolicy
from PyQt5.QtGui import QPixmap

from orangewidget import gui
from orangewidget.settings import Setting

from oasys.widgets import gui as oasysgui
from oasys.widgets.widget import OWWidget
from oasys.util.oasys_objects import OasysSurfaceData

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

from oasys.util.oasys_util import EmittingStream

from orangecontrib.esrf.syned.util.FEA_File import FEA_File
import orangecanvas.resources as resources
from silx.gui.plot import Plot2D

from srxraylib.metrology.profiles_simulation import slopes



class FiniteElementReader(OWWidget): #ow_automatic_element.AutomaticElement):

    name = "Surface / Finite Element reader"
    description = "Syned: Surface / Finite Element reader"
    icon = "icons/hhlo.png"
    maintainer = "M. Sanchez del Rio"
    maintainer_email = "srio@esrf.eu"
    priority = 1
    category = "Data File Tools"
    keywords = ["data", "file", "load", "read", "FEA", "Finite Elements"]

    outputs = [{"name": "Surface Data",
                "type": OasysSurfaceData,
                "doc": "Surface Data",
                "id": "Surface Data"},
               {"name": "DABAM 1D Profile",
                "type": numpy.ndarray,
                "doc": "numpy.ndarray",
                "id": "numpy.ndarray"},
               ]

    want_main_area = 1
    want_control_area = 1

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    IMAGE_WIDTH = 860
    IMAGE_HEIGHT = 645

    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT = 650

    file_in = Setting("")
    file_in_type = Setting(0)
    file_factor_x = Setting(1.0)
    file_factor_y = Setting(1.0)
    file_factor_z = Setting(1.0)

    file_in_skiprows = Setting(0)
    replicate_raw_data_flag = Setting(0)  # 0=None, 1=axis0, 2=axis1, 3=both axis
    # raw_render_option = Setting(2)

    file_out = Setting("") # copied from file_in and changed extension to h5
    n_axis_0 = Setting(801) #301)
    n_axis_1 = Setting(500) #51)

    detrended = Setting(0)
    detrended_fit_range = Setting(1.0)
    reset_height_method = Setting(2)
    remove_nan = Setting(0)
    invert_axes_names = Setting(1)
    extract_profile1D = Setting(0)
    coordinate_profile1D = Setting(0.0)
    sigma_flag = Setting(0)
    sigma_axis0 = Setting(10)
    sigma_axis1 = Setting(10)

    display_raw_data = Setting(0)

    fea_file_object = FEA_File()

    usage_path = os.path.join(resources.package_dirname("orangecontrib.esrf.syned.widgets.extension") , "misc", "finite_element_usage.png")

    def __init__(self, show_automatic_box=False):
        # super().__init__(show_automatic_box=show_automatic_box)
        super().__init__()

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width() * 0.05),
                               round(geom.height() * 0.05),
                               round(min(geom.width() * 0.98, self.MAX_WIDTH)),
                               round(min(geom.height() * 0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        #
        # tabs input panel
        #
        tabs_setting = oasysgui.tabWidget(self.controlArea)
        tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT)
        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)


        tab_calc = oasysgui.createTabPage(tabs_setting, "Calculate")
        tab_out = oasysgui.createTabPage(tabs_setting, "Output")
        tab_usa = oasysgui.createTabPage(tabs_setting, "Use of the Widget")
        #
        # tabs for results
        #
        self.tabs_setting = oasysgui.tabWidget(self.mainArea)
        self.tabs_setting.setFixedHeight(self.IMAGE_HEIGHT + 5)
        self.tabs_setting.setFixedWidth(self.IMAGE_WIDTH)
        self.create_tabs_results()



        #
        # parameters panel
        #

        gui.button(tab_calc, self, "Calculate Interpolated File", callback=self.calculate) #, height=45)

        # gui.separator(tab_calc, height=20)

        #
        #
        data_file_box = oasysgui.widgetBox(tab_calc, "Data file", addSpace=True,
                                         orientation="vertical",)


        figure_box = oasysgui.widgetBox(data_file_box, "", addSpace=True, orientation="horizontal") #width=550, height=50)
        self.le_beam_file_name = oasysgui.lineEdit(figure_box, self, "file_in", "FEA/Surface File/Url:",
                                                    labelWidth=140, valueType=str, orientation="horizontal")
        gui.button(figure_box, self, "...", callback=self.selectFile)


        data_file_box2 = oasysgui.widgetBox(data_file_box, "", addSpace=True, orientation="horizontal",)

        gui.comboBox(data_file_box2, self, "file_in_type", label="File content", labelWidth=220,
                     items=["ALS cols: A X Y Z DX DY DZ",
                            "ESRF cols: X,Y,Z,DX,DY,DZ",
                            "OASYS surface file [hdf5]"],
                     sendSelectedValue=False, orientation="horizontal")


        oasysgui.lineEdit(data_file_box2, self, "file_in_skiprows", "Skip rows:", labelWidth=300, valueType=int,
                          orientation="horizontal")

        data_file_expansion_box = oasysgui.widgetBox(data_file_box, "", addSpace=False,
                                                     orientation="horizontal")
        oasysgui.widgetLabel(data_file_expansion_box, label="Expansion factor")
        oasysgui.lineEdit(data_file_expansion_box, self, "file_factor_x", "X", labelWidth=10, controlWidth=35,
                          valueType=float, orientation="horizontal")
        oasysgui.lineEdit(data_file_expansion_box, self, "file_factor_y", "Y", labelWidth=10, controlWidth=35,
                          valueType=float, orientation="horizontal")
        oasysgui.lineEdit(data_file_expansion_box, self, "file_factor_z", "Z", labelWidth=10, controlWidth=35,
                          valueType=float, orientation="horizontal")


        gui.comboBox(data_file_box, self, "replicate_raw_data_flag", label="Replicate raw data", labelWidth=220,
                     items=["No","Along axis 0","Along axis 1","Along axes 0 and 1"],
                     sendSelectedValue=False, orientation="horizontal")

        #
        # interpolation
        #
        interpolation_box = oasysgui.widgetBox(tab_calc, "Interpolation", addSpace=True,
                                         orientation="vertical")


        interpolation_box2 = oasysgui.widgetBox(interpolation_box, "", addSpace=False,
                                         orientation="horizontal")

        oasysgui.lineEdit(interpolation_box2, self, "n_axis_0", "Pixels (axis 0)",
                          labelWidth=260, valueType=int, orientation="horizontal")

        oasysgui.lineEdit(interpolation_box2, self, "n_axis_1", "pixels (axis 1)",
                          labelWidth=260, valueType=int, orientation="horizontal")

        gui.comboBox(interpolation_box, self, "remove_nan", label="Remove interp NaN", labelWidth=220,
                     items=["No", "Yes (replace by min height)", "Yes (replace by zero)"],
                     sendSelectedValue=False, orientation="horizontal")




        #
        # post process
        #
        postprocess_box = oasysgui.widgetBox(tab_calc, "PostProcess", addSpace=True,
                                         orientation="vertical")



        gui.comboBox(postprocess_box, self, "detrended", label="Detrend profile", labelWidth=220,
                     items=["None", "Straight line (along axis 0)", "Straight line (along axis 1)",
                            "Best circle (along axis 0)", "Best circle (along axis 1)"],
                     sendSelectedValue=False, orientation="horizontal",
                     callback=self.set_visible)

        self.detrended_fit_range_id = oasysgui.widgetBox(postprocess_box, "", addSpace=True,
                                         orientation="vertical",)
        oasysgui.lineEdit(self.detrended_fit_range_id, self, "detrended_fit_range", "detrend fit up to [m]",
                          labelWidth=220, valueType=float, orientation="horizontal")

        gui.comboBox(postprocess_box, self, "reset_height_method", label="Reset zero height", labelWidth=220,
                     items=["No", "To height minimum", "To center"],
                     sendSelectedValue=False, orientation="horizontal")


        gui.comboBox(postprocess_box, self, "sigma_flag", label="Gaussian filter", labelWidth=220,
                     items=["None", "Yes"],
                     sendSelectedValue=False, orientation="horizontal",
                     callback=self.set_visible)

        self.sigma_id = oasysgui.widgetBox(postprocess_box, "", addSpace=True, orientation="horizontal",)


        oasysgui.widgetLabel(self.sigma_id, label="Gaussian sigma [pixels] axis",labelWidth=350)
        oasysgui.lineEdit(self.sigma_id, self, "sigma_axis0", "0:",
                          labelWidth=0, controlWidth=50, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.sigma_id, self, "sigma_axis1", "1:",
                          labelWidth=0, controlWidth=50, valueType=float, orientation="horizontal")




        gui.comboBox(postprocess_box, self, "invert_axes_names", label="Invert axes", labelWidth=120,
                     items=['No','Yes'],
                     sendSelectedValue=False, orientation="horizontal")

        #
        # output tab
        #

        profile1D_box = oasysgui.widgetBox(tab_out, "1D profile", addSpace=True,
                                         orientation="vertical",)
        gui.comboBox(profile1D_box, self, "extract_profile1D", label="Extract and send 1D profile", labelWidth=220,
                     items=["axis 0 (horizontal)", "axis 1 (vertical)"],
                     sendSelectedValue=False, orientation="horizontal")

        oasysgui.lineEdit(profile1D_box, self, "coordinate_profile1D", "At coordinate [m]:", labelWidth=260, valueType=float,
                          orientation="horizontal")

        gui.separator(tab_out, height=20)


        display_box = oasysgui.widgetBox(tab_out, "Display", addSpace=True,
                                         orientation="vertical",)
        gui.comboBox(display_box, self, "display_raw_data", label="Display Raw data [slow]", labelWidth=220,
                     items=["No", "Yes"],
                     sendSelectedValue=False, orientation="horizontal")

        oasysgui.lineEdit(profile1D_box, self, "coordinate_profile1D", "At coordinate [m]:", labelWidth=260, valueType=float,
                          orientation="horizontal")

        gui.separator(tab_out, height=20)






        file_info_box = oasysgui.widgetBox(tab_out, "Info", addSpace=True,
                                         orientation="vertical",)

        tmp = oasysgui.lineEdit(file_info_box, self, "file_out", "Output file name",
                          labelWidth=150, valueType=str, orientation="horizontal")
        tmp.setEnabled(False)

        #
        # usage
        #
        tab_usa.setStyleSheet("background-color: white;")

        usage_box = oasysgui.widgetBox(tab_usa, "", addSpace=True, orientation="horizontal")

        label = QLabel("")
        label.setAlignment(Qt.AlignCenter)
        label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        label.setPixmap(QPixmap(self.usage_path))

        usage_box.layout().addWidget(label)

        self.set_visible()

    def set_visible(self):
        if self.detrended == 0:
            self.detrended_fit_range_id.setVisible(False)
        else:
            self.detrended_fit_range_id.setVisible(True)

        if self.sigma_flag == 0:
            self.sigma_id.setVisible(False)
        else:
            self.sigma_id.setVisible(True)

    def create_tabs_results(self):

        tabs_setting = self.tabs_setting
        tmp = oasysgui.createTabPage(tabs_setting, "Result")
        self.result_id = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.result_id.setFixedHeight(self.IMAGE_HEIGHT - 30)
        self.result_id.setFixedWidth(self.IMAGE_WIDTH - 20)

        tmp = oasysgui.createTabPage(tabs_setting, "Interpolation")
        self.interpolation_id = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.interpolation_id.setFixedHeight(self.IMAGE_HEIGHT - 30)
        self.interpolation_id.setFixedWidth(self.IMAGE_WIDTH - 20)

        tmp = oasysgui.createTabPage(tabs_setting, "Triangulation")
        self.triangulation_id = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.triangulation_id.setFixedHeight(self.IMAGE_HEIGHT - 30)
        self.triangulation_id.setFixedWidth(self.IMAGE_WIDTH - 20)

        tmp = oasysgui.createTabPage(tabs_setting, "Raw Data")
        self.rawdata_id = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.rawdata_id.setFixedHeight(self.IMAGE_HEIGHT - 30)
        self.rawdata_id.setFixedWidth(self.IMAGE_WIDTH - 20)

        tmp = oasysgui.createTabPage(tabs_setting, "1D profile")
        self.profile1D_id = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.profile1D_id.setFixedHeight(self.IMAGE_HEIGHT - 30)
        self.profile1D_id.setFixedWidth(self.IMAGE_WIDTH - 20)

        tmp = oasysgui.createTabPage(tabs_setting, "1D slope")
        self.slope1D_id = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.slope1D_id.setFixedHeight(self.IMAGE_HEIGHT - 30)
        self.slope1D_id.setFixedWidth(self.IMAGE_WIDTH - 20)

        #
        tmp = oasysgui.createTabPage(tabs_setting, "Output")
        self.info_id = oasysgui.textArea(height=self.IMAGE_HEIGHT - 35)
        info_box = oasysgui.widgetBox(tmp, "", addSpace=True, orientation="horizontal",
                                      height=self.IMAGE_HEIGHT - 20, width=self.IMAGE_WIDTH - 20)
        info_box.layout().addWidget(self.info_id)

    def set_input_file(self,filename):
        self.le_beam_file_name.setText(filename)

    def selectFile(self):
        filename = oasysgui.selectFileFromDialog(self,
                previous_file_path=self.file_in, message="Open FEA File",
                start_directory=".", file_extension_filter="*.*")

        self.le_beam_file_name.setText(filename)
        self.set_file_out()

    def load_raw_data(self):
        self.fea_file_object = FEA_File()
        self.fea_file_object.set_filename(self.file_in)

        self.fea_file_object.load_multicolumn_file(skiprows=self.file_in_skiprows,
                                                   file_in_type=self.file_in_type,
                                                   factorX=self.file_factor_x,
                                                   factorY=self.file_factor_y,
                                                   factorZ=self.file_factor_z)

        self.fea_file_object.replicate_raw_data(self.replicate_raw_data_flag)

    def writeStdOut(self, text="", initialize=False):
        cursor = self.info_id.textCursor()
        if initialize:
            self.info_id.setText(text)
        else:
            cursor.movePosition(QtGui.QTextCursor.End)
            cursor.insertText(text)

    def set_file_out(self):
        if self.file_in_type == 2:
            file_out = os.path.splitext(self.file_in)[0] + '_processed.h5'
        else:
            file_out = os.path.splitext(self.file_in)[0]+'.h5'
        # if self.file_out[0:4] == "http":
        #     self.file_out = "tmp.h5" # self.file_out.split(os.sep)[-1]
        # print(">>>>>>>>>>>>>>>>>>>",self.file_out)

        if file_out[0:4] == "http":
            file_out = file_out.split(os.sep)[-1]

        self.file_out = file_out

    def calculate(self):
        self.writeStdOut(initialize=True)

        # sys.stdout = EmittingStream(textWritten=self.writeStdOut)
        sys.stdout = EmittingStream(textWritten=self.writeStdOut)

        self.load_raw_data()

        self.fea_file_object.triangulate()


        if self.file_in_type == 2:
            self.fea_file_object.interpolate(self.n_axis_0, self.n_axis_1, remove_nan=self.remove_nan)
        else:
            # add 3 pixels to match requested pixels after edge removal
            self.fea_file_object.interpolate(self.n_axis_0 + 3, self.n_axis_1 + 3, remove_nan=self.remove_nan)

        if self.fea_file_object.does_interpolated_have_nan():
            self.fea_file_object.remove_borders_in_interpolated_data()

        if self.detrended == 0:
            pass
        elif self.detrended == 1:
            self.fea_file_object.detrend_straight_line(axis=0, fitting_domain_ratio=self.detrended_fit_range)
        elif self.detrended == 2:
            self.fea_file_object.detrend_straight_line(axis=1, fitting_domain_ratio=self.detrended_fit_range)
        elif self.detrended == 3:
            self.fea_file_object.detrend_best_circle(axis=0, fitting_domain_ratio=self.detrended_fit_range)
        elif self.detrended == 4:
            self.fea_file_object.detrend_best_circle(axis=1, fitting_domain_ratio=self.detrended_fit_range)


        if self.reset_height_method == 0:
            pass
        elif self.reset_height_method == 1:
            self.fea_file_object.reset_height_to_minimum()
        elif self.reset_height_method == 2:
            self.fea_file_object.reset_height_to_central_value()

        if self.sigma_flag == 1:
            self.fea_file_object.gaussian_filter(sigma_axis0=self.sigma_axis0,sigma_axis1=self.sigma_axis1)


        self.set_file_out()
        self.fea_file_object.write_h5_surface(filename=self.file_out, invert_axes_names=self.invert_axes_names)

        print("File %s written to disk.\n" % self.file_out)

        self.plot_and_send_results()


    def plot_and_send_results(self):
        #
        # result
        #
        if self.invert_axes_names:
            self.plot_data2D(self.fea_file_object.Z_INTERPOLATED,
                           self.fea_file_object.x_interpolated,
                           self.fea_file_object.y_interpolated,self.result_id,
                       title="file: %s, axes names INVERTED from ANSYS"%self.file_in,
                       xtitle="Y [m] (%d pixels, max:%f)"%(self.fea_file_object.x_interpolated.size,
                                                       self.fea_file_object.x_interpolated.max()),
                       ytitle="X [m] (%d pixels, max:%f)"%(self.fea_file_object.y_interpolated.size,
                                                       self.fea_file_object.y_interpolated.max()) )
        else:
            self.plot_data2D(self.fea_file_object.Z_INTERPOLATED,
                           self.fea_file_object.x_interpolated,
                           self.fea_file_object.y_interpolated,self.result_id,
                       title="file: %s, axes as in ANSYS"%self.file_in,
                       xtitle="X [m] (%d pixels, max:%f)"%(self.fea_file_object.x_interpolated.size,
                                                       self.fea_file_object.x_interpolated.max()),
                       ytitle="Y [m] (%d pixels, max:%f)"%(self.fea_file_object.y_interpolated.size,
                                                       self.fea_file_object.y_interpolated.max()) )

        slp = slopes(self.fea_file_object.Z_INTERPOLATED,
                     self.fea_file_object.x_interpolated,
                     self.fea_file_object.y_interpolated,
                     silent=0, return_only_rms=0)


        print("\n\n\n**** heights: ****")
        print("Heigh error StDev: %g um" % (1e6 * self.fea_file_object.Z_INTERPOLATED.std()))
        print("*****************")

        #
        # interpolation plot
        #
        if self.file_in_type != 2:
            self.interpolation_id.layout().removeItem(self.interpolation_id.layout().itemAt(1))
            self.interpolation_id.layout().removeItem(self.interpolation_id.layout().itemAt(0))

            f = self.fea_file_object.plot_interpolated(show=0)
            figure_canvas = FigureCanvasQTAgg(f)
            toolbar = NavigationToolbar(figure_canvas, self)

            self.interpolation_id.layout().addWidget(toolbar)
            self.interpolation_id.layout().addWidget(figure_canvas)


        #
        # triangulation plot
        #
        if self.file_in_type != 2:
            self.triangulation_id.layout().removeItem(self.triangulation_id.layout().itemAt(1))
            self.triangulation_id.layout().removeItem(self.triangulation_id.layout().itemAt(0))

            f = self.fea_file_object.plot_triangulation(show=0)
            figure_canvas = FigureCanvasQTAgg(f)
            toolbar = NavigationToolbar(figure_canvas, self)

            self.triangulation_id.layout().addWidget(toolbar)
            self.triangulation_id.layout().addWidget(figure_canvas)

        #
        # raw data
        #
        if self.display_raw_data:
            self.rawdata_id.layout().removeItem(self.rawdata_id.layout().itemAt(1))
            self.rawdata_id.layout().removeItem(self.rawdata_id.layout().itemAt(0))

            xs, ys, zs = self.fea_file_object.get_deformed()

            xs *= 1e3
            ys *= 1e3
            zs *= 1e6

            fig = Figure()
            self.axis = fig.add_subplot(111, projection='3d')

            # For each set of style and range settings, plot n random points in the box
            # defined by x in [23, 32], y in [0, 100], z in [zlow, zhigh].
            # for m, zlow, zhigh in [('o', -50, -25), ('^', -30, -5)]:
            for m, zlow, zhigh in [('o', zs.min(), zs.max())]:
                self.axis.scatter(xs, ys, zs, marker=m)

            self.axis.set_xlabel('X [mm]')
            self.axis.set_ylabel('Y [mm]')
            self.axis.set_zlabel('Z [um]')


            figure_canvas = FigureCanvasQTAgg(fig)
            toolbar = NavigationToolbar(figure_canvas, self)

            self.rawdata_id.layout().addWidget(toolbar)
            self.rawdata_id.layout().addWidget(figure_canvas)

            self.axis.mouse_init()

        #
        # result
        #

        mesh = self.fea_file_object.Z_INTERPOLATED
        mesh_shape = mesh.shape
        x = self.fea_file_object.x_interpolated
        y = self.fea_file_object.y_interpolated

        if self.extract_profile1D == 0:
            abscissas = x
            perp_abscissas = y
            index0 = numpy.argwhere(perp_abscissas >= self.coordinate_profile1D)
            try:
                index0 = index0[0][0]
            except:
                index0 = -1
            profile1D = mesh[:, index0]
            slope1D = numpy.gradient(profile1D,abscissas)
            profile1D_std = profile1D.std()
            slope1D_std = slope1D.std()
            if self.invert_axes_names:
                title = "profile at X[%d] = %g; StDev = %g um" % (index0, perp_abscissas[index0], 1e6*profile1D_std)
                titleS = "slopes at X[%d] = %g; StDev = %g urad" % (index0, perp_abscissas[index0], 1e6*slope1D_std)
                xtitle = "Y [m] "
            else:
                title = "profile at Y[%d] = %g; StDev = %g um" % (index0, perp_abscissas[index0], 1e6*profile1D_std)
                titleS = "slopes at Y[%d] = %g; StDev = %g urad" % (index0, perp_abscissas[index0], 1e6*slope1D_std)
                xtitle = "X [m] "
            self.plot_data1D(abscissas, 1e6*profile1D, self.profile1D_id, title=title, xtitle=xtitle, ytitle="Z [um] ")
            self.plot_data1D(abscissas, 1e6*slope1D, self.slope1D_id, title=titleS, xtitle=xtitle, ytitle="Z' [urad]")
        else:
            abscissas = y
            perp_abscissas = x
            index0 = numpy.argwhere(perp_abscissas >= self.coordinate_profile1D)
            try:
                index0 = index0[0][0]
            except:
                index0 = -1
            profile1D = mesh[index0, :]
            slope1D = numpy.gradient(profile1D,abscissas)
            profile1D_std = profile1D.std()
            slope1D_std = slope1D.std()
            if self.invert_axes_names:
                title = "profile at Y[%d] = %g; StDev = %g um" % (index0, perp_abscissas[index0], 1e6*profile1D_std)
                titleS = "slopes at Y[%d] = %g; StDev = %g urad" % (index0, perp_abscissas[index0], 1e6*slope1D_std)
                xtitle = "X [m] "
            else:
                title = "profile at X[%d] = %g; StDev = %g um" % (index0, perp_abscissas[index0], 1e6*profile1D_std)
                titleS = "slopes at X[%d] = %g; StDev = %g urad" % (index0, perp_abscissas[index0], 1e6*slope1D_std)
                xtitle = "Y [m] "
            self.plot_data1D(abscissas, 1e6*profile1D, self.profile1D_id, title=title, xtitle=xtitle, ytitle="Z [um] ")
            self.plot_data1D(abscissas, 1e6*slope1D, self.slope1D_id, title=titleS, xtitle=xtitle, ytitle="Z' [urad]")

        if self.invert_axes_names:
            self.send("Surface Data",
                      OasysSurfaceData(xx=self.fea_file_object.y_interpolated,
                                       yy=self.fea_file_object.x_interpolated,
                                       zz=self.fea_file_object.Z_INTERPOLATED,
                                       surface_data_file=self.file_out))
        else:
            self.send("Surface Data",
                      OasysSurfaceData(xx=self.fea_file_object.x_interpolated,
                                       yy=self.fea_file_object.y_interpolated,
                                       zz=self.fea_file_object.Z_INTERPOLATED.T,
                                       surface_data_file=self.file_out))

        dabam_profile = numpy.zeros((profile1D.size, 2))
        dabam_profile[:, 0] = abscissas
        dabam_profile[:, 1] = profile1D
        self.send("DABAM 1D Profile", dabam_profile)



    def plot_data2D(self, data2D, dataX, dataY, tabs_canvas_index, title="title", xtitle="X",ytitle="Y"):


        tabs_canvas_index.layout().removeItem(tabs_canvas_index.layout().itemAt(0))

        origin = (dataX[0], dataY[0])
        scale = (dataX[1] - dataX[0], dataY[1] - dataY[0])

        colormap = {"name": "temperature", "normalization": "linear",
                    "autoscale": True, "vmin": 0, "vmax": 0, "colors": 256}

        tmp = Plot2D()
        tmp.resetZoom()
        tmp.setXAxisAutoScale(True)
        tmp.setYAxisAutoScale(True)
        tmp.setGraphGrid(False)
        tmp.setKeepDataAspectRatio(True)
        tmp.yAxisInvertedAction.setVisible(False)
        tmp.setXAxisLogarithmic(False)
        tmp.setYAxisLogarithmic(False)
        tmp.getMaskAction().setVisible(False)
        tmp.getRoiAction().setVisible(False)
        tmp.getColormapAction().setVisible(True)
        tmp.setKeepDataAspectRatio(False)
        tmp.addImage(data2D.T,legend="1",scale=scale,origin=origin,colormap=colormap,replace=True)
        tmp.setActiveImage("1")
        tmp.setGraphXLabel(xtitle)
        tmp.setGraphYLabel(ytitle)
        tmp.setGraphTitle(title)
        tabs_canvas_index.layout().addWidget(tmp)

    def plot_data1D(self, dataX, dataY, tabs_canvas_index, title="", xtitle="", ytitle=""):

        tabs_canvas_index.layout().removeItem(tabs_canvas_index.layout().itemAt(0))

        tmp = oasysgui.plotWindow()
        tmp.addCurve(dataX, dataY)
        tmp.resetZoom()
        tmp.setXAxisAutoScale(True)
        tmp.setYAxisAutoScale(True)
        tmp.setGraphGrid(False)
        tmp.setXAxisLogarithmic(False)
        tmp.setYAxisLogarithmic(False)
        tmp.setGraphXLabel(xtitle)
        tmp.setGraphYLabel(ytitle)
        tmp.setGraphTitle(title)

        tabs_canvas_index.layout().addWidget(tmp)


if __name__ == "__main__":
    import sys



    a = QApplication(sys.argv)
    ow = FiniteElementReader()

    # ALS
    # ow.set_input_file("/users/srio/Oasys/s4.txt")
    # ow.file_in_type = 0
    # ow.replicate_raw_data_flag = 0

    # ESRF
    # ow.set_input_file("https://raw.githubusercontent.com/oasys-esrf-kit/OasysWorkspaces/master/ANSYS/ID03-DMM-LC3-E20.25keV-th0.245deg-Wc8mm.out")
    # ow.file_in_type = 1
    # ow.file_in_skiprows = 9
    # ow.replicate_raw_data_flag = 1
    # ow.file_factor_x = 1e-3
    # ow.file_factor_y = 1e-3
    # ow.file_factor_z = 1e-3



    # OASYS hdf5
    # ow.set_input_file("/users/srio/OASYS1.2/shadow3-scripts/METROLOGY/ring256.h5")
    # ow.set_input_file("/users/srio/Downloads/dabam2d-001.h5")
    ow.set_input_file("https://raw.githubusercontent.com/srio/dabam2d/main/data/dabam2d-001.h5")
    ow.file_in_type = 2
    ow.n_axis_0 = 0
    ow.n_axis_1 = 0
    ow.invert_axes_names = 0
    ow.extract_profile1D = 1



    ow.show()
    a.exec_()
    ow.saveSettings()


    from srxraylib.plot.gol import plot_image, set_qt, plot
    # import os
    #
    # import numpy
    # example_file = "https://raw.githubusercontent.com/oasys-esrf-kit/OasysWorkspaces/master/ANSYS/ID03-DMM-LC3-E20.25keV-th0.245deg-Wc8mm.out"
    #
    # node = numpy.loadtxt(example_file, skiprows=9, delimiter=',', dtype=numpy.float64)
    # print(">>>",node.shape)
    #

    # o1 = FEA_File.process_file(example_file, n_axis_0=301, n_axis_1=51,
    #              filename_out=example_file+".h5", invert_axes_names=True,
    #              detrend=False, reset_height_method=0, skiprows=9, file_in_type=1, do_plot=True)



    # o1 = FEA_File.process_file("73water_side_cooled_notches_best_LH.txt", n_axis_0=1001, n_axis_1=101,
    #              filename_out="/home/manuel/Oasys/water_side_cooled_notches_best_LH.h5", invert_axes_names=True,
    #              detrend=True, reset_height_method=2,
    #              replicate_raw_data_flag=3,do_plot=False)

    # o1 = FEA_File.process_file("73water_side_cooled_notches_best_LV.txt", n_axis_0=1001, n_axis_1=101,
    #              filename_out="/home/manuel/Oasys/water_side_cooled_notches_best_LV.h5", invert_axes_names=True,
    #              detrend=False, reset_height_method=0,
    #              replicate_raw_data_flag=3,do_plot=False)

    #
    # o1.plot_triangulation()
    # o1.plot_interpolated()
    # o1.plot_surface_image()


